/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef SMSVDP_H
#define SMSVDP_H

typedef struct smscol_s	{
	/* ARGB8888 color */
	unsigned char a;
	unsigned char r;
	unsigned char g;
	unsigned char b;
} sms_vdp_color_t;

/* Pattern structure */
typedef struct smspat_s	{
	/* Preconverted texture */
	unsigned char texture[64];
	
	/* Offset into VRAM */
	unsigned short offset;
	
	/* Dirty flag */
	unsigned char dirty;
} sms_vdp_pattern_t;

typedef struct smsvdp_s	{
	/* Command Word - written to the control port
	   Consists of a 2-bit code and a 14-bit address */
	unsigned char code;
	unsigned short addr;

	/* Read buffer - used for reads from VRAM */
	unsigned char read_buf;

	/* Software palette, stored both in SMS and
	   native formats (CRAM) */
	unsigned char *cram;
	sms_vdp_color_t *pal;

	/* Status flags */
	unsigned char status;

	/* VDP Registers */
	unsigned char regs[0x0F];

	/* VRAM */
	unsigned char *vram;

	/* H/V Counters */
	unsigned short hcnt;
	unsigned char vcnt;

	/* Line interrupt counter */
	unsigned char linecnt;

	/* Internal flags */
	unsigned int flags;

	/* Preconverted patterns */
	sms_vdp_pattern_t pattern[512];

	/* Background priority levels */
	unsigned char bg_prio[32][32];

	/* Internal Framebuffer - 256x256 pixels, 16 or 32bpp */
	unsigned char *framebuffer;
} sms_vdp_t;

void sms_vdp_data_write(unsigned char data);
void sms_vdp_ctl_write(unsigned char data);

unsigned char sms_vdp_hcnt_read();
unsigned char sms_vdp_vcnt_read();
unsigned char sms_vdp_data_read();
unsigned char sms_vdp_status_read();

void sms_vdp_execute(int line);

int sms_vdp_init();
int sms_vdp_shutdown();

#endif /* !SMSVDP_H */
