/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

switch(op)	{
	case 0x09: /* ADD IX, BC */
	{
		uint32 tmp;
		tmp = cpu->ix.w + cpu->bc.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->ix.w ^ tmp ^ cpu->bc.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->ix.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x19: /* ADD IX, DE */
	{
		uint32 tmp;
		tmp = cpu->ix.w + cpu->de.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->ix.w ^ tmp ^ cpu->de.w) >> 8) & 0x10) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->ix.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x21: /* LD IX, nn */
	{
		cpu->ix.b.l = cpu->mread(cpu->pc.w++);
		cpu->ix.b.h = cpu->mread(cpu->pc.w++);
		cpu->cycles += 14;
		break;
	}
	case 0x22: /* LD (nn), IX */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->mwrite(tmp, cpu->ix.b.l);
		cpu->mwrite(tmp + 1, cpu->ix.b.h);
		cpu->cycles += 20;
		break;
	}
	case 0x23: /* INC IX */
	{
		++cpu->ix.w;
		cpu->cycles += 10;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x24: /* INC IXh */
	{
		break;
	}
	case 0x25: /* DEC IXh */
	{
		break;
	}
	case 0x26: /* LD IXh, n */
	{
		break;
	}
#endif
	case 0x29: /* ADD IX, IX */
	{
		uint32 tmp;
		tmp = cpu->ix.w + cpu->ix.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->ix.w ^ tmp ^ cpu->ix.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->ix.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x2A: /* LD IX, (nn) */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->ix.b.l = cpu->mread(tmp);
		cpu->ix.b.h = cpu->mread(tmp + 1);
		cpu->cycles += 20;
		break;
	}
	case 0x2B: /* DEC IX */
	{
		--cpu->ix.w;
		cpu->cycles += 10;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x2C: /* INC IXl */
	{
		break;
	}
	case 0x2D: /* DEC IXl */
	{
		break;
	}
	case 0x2E: /* LD IXl, n */
	{
		break;
	}
#endif
	case 0x34: /* INC (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp);
		uint32 tmp2 = m + 1;
		cpu->af.b.l = ZStable[tmp2 & 0xFF] | /* S, Z */
			((tmp2 & 0x0F) ? 0x00 : 0x10) | /* H */
			((m == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->mwrite(cpu->ix.w + tmp, tmp2 & 0xFF);
		cpu->cycles += 23;
		break;
	}
	case 0x35: /* DEC (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp);
		uint32 tmp2 = m - 1;
		cpu->af.b.l = ZStable[tmp2 & 0xFF] | /* S, Z */
			((m ^ tmp2) & 0x10) | /* H */
			(m == 0x80 ? 0x04 : 0x00) | /* P */
			0x02 | /* N */
			(cpu->af.b.l & 0x01); /* C */
		cpu->mwrite(cpu->ix.w + tmp, tmp2 & 0xFF);
		cpu->cycles += 23;
		break;
	}
	case 0x36: /* LD (IX+d), n */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->pc.w++);
		cpu->mwrite(tmp + cpu->ix.w, m);
		cpu->cycles += 19;
		break;
	}
	case 0x39: /* ADD IX, SP */
	{
		uint32 tmp;
		tmp = cpu->ix.w + cpu->sp.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->ix.w ^ tmp ^ cpu->sp.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->ix.w = tmp;
		cpu->cycles += 15;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x44: /* LD B, IXh */
	{
		break;
	}
	case 0x45: /* LD B, IXl */
	{
		break;
	}
#endif
	case 0x46: /* LD B, (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->bc.b.h = cpu->mread(cpu->ix.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x4C: /* LD C, IXh */
	{
		break;
	}
	case 0x4D: /* LD C, IXl */
	{
		break;
	}
#endif
	case 0x4E: /* LD C, (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->bc.b.l = cpu->mread(cpu->ix.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x54: /* LD D, IXh */
	{
		break;
	}
	case 0x55: /* LD D, IXl */
	{
		break;
	}
#endif
	case 0x56: /* LD D, (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->de.b.h = cpu->mread(cpu->ix.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x5C: /* LD E, IXh */
	{
		break;
	}
	case 0x5D: /* LD E, IXl */
	{
		break;
	}
#endif
	case 0x5E: /* LD E, (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->de.b.l = cpu->mread(cpu->ix.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x60: /* LD IXh, B */
	{
		break;
	}
	case 0x61: /* LD IXh, C */
	{
		break;
	}
	case 0x62: /* LD IXh, D */
	{
		break;
	}
	case 0x63: /* LD IXh, E */
	{
		break;
	}
	case 0x64: /* LD IXh, IXh */
	{
		break;
	}
	case 0x65: /* LD IXh, IXl */
	{
		break;
	}
#endif
	case 0x66: /* LD H, (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->hl.b.h = cpu->mread(cpu->ix.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x67: /* LD IXh, A */
	{
		break;
	}
	case 0x68: /* LD IXl, B */
	{
		break;
	}
	case 0x69: /* LD IXl, C */
	{
		break;
	}
	case 0x6A: /* LD IXl, D */
	{
		break;
	}
	case 0x6B: /* LD IXl, E */
	{
		break;
	}
	case 0x6C: /* LD IXl, IXh */
	{
		break;
	}
	case 0x6D: /* LD IXl, IXl */
	{
		break;
	}
#endif
	case 0x6E: /* LD L, (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->hl.b.l = cpu->mread(cpu->ix.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x6F: /* LD IXl, A */
	{
		break;
	}
#endif
	case 0x70: /* LD (IX+d), B */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->ix.w + tmp, cpu->bc.b.h);
		cpu->cycles += 19;
		break;
	}
	case 0x71: /* LD (IX+d), C */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->ix.w + tmp, cpu->bc.b.l);
		cpu->cycles += 19;
		break;
	}
	case 0x72: /* LD (IX+d), D */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->ix.w + tmp, cpu->de.b.h);
		cpu->cycles += 19;
		break;
	}
	case 0x73: /* LD (IX+d), E */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->ix.w + tmp, cpu->de.b.l);
		cpu->cycles += 19;
		break;
	}
	case 0x74: /* LD (IX+d), H */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->ix.w + tmp, cpu->hl.b.h);
		cpu->cycles += 19;
		break;
	}
	case 0x75: /* LD (IX+d), L */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->ix.w + tmp, cpu->hl.b.l);
		cpu->cycles += 19;
		break;
	}
	case 0x77: /* LD (IX+d), A */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->ix.w + tmp, cpu->af.b.h);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x7C: /* LD A, IXh */
	{
		break;
	}
	case 0x7D: /* LD A, IXl */
	{
		break;
	}
#endif
	case 0x7E: /* LD A, (IX+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->af.b.h = cpu->mread(cpu->ix.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x84: /* ADD A, IXh */
	{
		break;
	}
	case 0x85: /* ADD A, IXl */
	{
		break;
	}
#endif
	case 0x86: /* ADD A, (IX+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp2);
		tmp = cpu->af.b.h + m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h ^ 0x80) & (m ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp & 0xFF;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x8C: /* ADC A, IXh */
	{
		break;
	}
	case 0x8D: /* ADC A, IXl */
	{
		break;
	}
#endif
	case 0x8E: /* ADC A, (IX+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp2);
		tmp = cpu->af.b.h + m + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h ^ 0x80) & (m ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp & 0xFF;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x94: /* SUB A, IXh */
	{
		break;
	}
	case 0x95: /* SUB A, IXl */
	{
		break;
	}
#endif
	case 0x96: /* SUB A, (IX+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp2);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			((tmp >> 8) & 0x01) | /* Set the carry flag */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* Overflow */
			((m ^ cpu->af.b.h ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->af.b.h = tmp & 0xFF;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x9C: /* SBC A, IXh */
	{
		break;
	}
	case 0x9D: /* SBC A, IXl */
	{
		break;
	}
#endif
	case 0x9E: /* SBC A, (IX+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp2);
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (m ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp & 0xFF;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0xA4: /* AND A, IXh */
	{
		break;
	}
	case 0xA5: /* AND A, IXl */
	{
		break;
	}
#endif
	case 0xA6: /* AND A, (IX+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp2);
		tmp = cpu->af.b.h & m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			0x10; /* Set the Half-carry flag */
		cpu->af.b.h = tmp;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0xAC: /* XOR A, IXh */
	{
		break;
	}
	case 0xAD: /* XOR A, IXl */
	{
		break;
	}
#endif
	case 0xAE: /* XOR A, (IX+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp2);
		tmp = cpu->af.b.h ^ m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* Set the Zero/Sign flags */
		cpu->af.b.h = tmp;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0xB4: /* OR A, IXh */
	{
		break;
	}
	case 0xB5: /* OR A, IXl */
	{
		break;
	}
#endif
	case 0xB6: /* OR A, (IX+d) */
	{
		uint32 tmp;
		int8 tmp2 = cpu->mread(cpu->pc.w++);
        uint8 m = cpu->mread(cpu->ix.w + tmp2);
		tmp = cpu->af.b.h | m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* Set the Zero/Sign flags */
		cpu->af.b.h = tmp;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0xBC: /* CP A, IXh */
	{
		break;
	}
	case 0xBD: /* CP A, IXl */
	{
		break;
	}
#endif
	case 0xBE: /* CP A, (IX+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + tmp2);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 19;
		break;
	}
	case 0xCB: /* DDCB prefixed opcodes */
	{
		uint8 op2 = cpu->mread(cpu->pc.w + 1);

#		include "z80_switch_DDCB.h"

		++cpu->pc.w;

		break;
	}
	case 0xE1: /* POP IX */
	{
		cpu->ix.b.l = cpu->mread(cpu->sp.w++);
		cpu->ix.b.h = cpu->mread(cpu->sp.w++);
		cpu->cycles += 14;
		break;
	}
	case 0xE3: /* EX (SP), IX */
	{
		uint8 tmp, tmp2;
		tmp = cpu->ix.b.l;
		tmp2 = cpu->ix.b.h;
		cpu->ix.b.l = cpu->mread(cpu->sp.w++);
		cpu->ix.b.h = cpu->mread(cpu->sp.w);
		cpu->mwrite(cpu->sp.w, tmp2);
		cpu->mwrite(--cpu->sp.w, tmp);
		cpu->cycles += 23;
		break;
	}
	case 0xE5: /* PUSH IX */
	{
		cpu->mwrite(--cpu->sp.w, cpu->ix.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->ix.b.l);
		cpu->cycles += 15;
		break;
	}
	case 0xE9: /* JP (IX) (Really JP IX) */
	{
		cpu->pc.w = cpu->ix.w;
		cpu->cycles += 8;
		break;
	}
	case 0xF9: /* LD SP, IX */
	{
		cpu->sp.w = cpu->ix.w;
		cpu->cycles += 10;
		break;
	}
	default:
	{
		printf("Error in emulation, bailing out, opcode = 0xDD%02X\n", op);
		assert(0);
	}
}
