/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#import "EmuController.h"
#include "sms.h"
#include "smsmem.h"
#include "sn76489.h"

/* From main.m */
extern void emu_loop();

/* from sms.c */
extern int sms_psg_enabled;
extern sn76489_t psg;
extern int sms_running;

@implementation EmuController

- (IBAction)enableNoise:(id)sender
{
	if([noise state] == NSOnState)	{
		[noise setState: NSOffState];
		psg.enabled_channels &= ~ENABLE_NOISE;
	}
	else	{
		[noise setState: NSOnState];
		psg.enabled_channels |= ENABLE_NOISE;
	}
}

- (IBAction)enablePSG:(id)sender
{
	if([PSG state] == NSOnState)	{
		[PSG setState: NSOffState];
		sms_psg_enabled = 0;
	}
	else	{
		[PSG setState: NSOnState];
		sms_psg_enabled = 1;
	}
}

- (IBAction)enableTone0:(id)sender
{
	if([tone0 state] == NSOnState)	{
		[tone0 setState: NSOffState];
		psg.enabled_channels &= ~ENABLE_TONE0;
	}
	else	{
		[tone0 setState: NSOnState];
		psg.enabled_channels |= ENABLE_TONE0;
	}
}

- (IBAction)enableTone1:(id)sender
{
	if([tone1 state] == NSOnState)	{
		[tone1 setState: NSOffState];
		psg.enabled_channels &= ~ENABLE_TONE1;
	}
	else	{
		[tone1 setState: NSOnState];
		psg.enabled_channels |= ENABLE_TONE1;
	}
}

- (IBAction)enableTone2:(id)sender
{
	if([tone2 state] == NSOnState)	{
		[tone2 setState: NSOffState];
		psg.enabled_channels &= ~ENABLE_TONE2;
	}
	else	{
		[tone2 setState: NSOnState];
		psg.enabled_channels |= ENABLE_TONE2;
	}
}

- (IBAction)loadMasterRom:(id)sender
{
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	[openPanel setTitle: @"Choose a Master System ROM"];
	[openPanel setCanChooseDirectories: NO];

    if(sms_running == 1)    {
        sms_shutdown();
    }
	
	if([openPanel runModal] == NSOKButton)	{
		NSString *filePath = [openPanel filename];

		sms_init();
		sms_mem_load_rom([filePath UTF8String]);
	}
	started = 0;
}

- (IBAction)startMasterSys:(id)sender
{
	if(!started)	{
		[masterStart setState:NSOnState];
		[NSThread detachNewThreadSelector:@selector(masterEmuThread:) toTarget:self withObject:nil];
		started = 1;
	}
}

- (void)masterEmuThread:(id)junk
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	emu_loop();

	sms_shutdown();
	[pool release];
}

@end
