/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#import "EmuView.h"

#include "sms.h"
#include "smsvdp.h"

extern EmuView *theview;
extern sms_vdp_t smsvdp;

@implementation EmuView

- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil) {
		theview = self;
	}
	return self;
}

- (void)drawRect:(NSRect)rect
{
	CGContextRef cgContext;
	CGImageRef cgImage;
	CGColorSpaceRef cgSpace;
	CGDataProviderRef cgData;
	CGRect cgRect;
	unsigned char *bitmap;

	cgContext = [[NSGraphicsContext currentContext] graphicsPort];
	bitmap = smsvdp.framebuffer;
	cgData = CGDataProviderCreateWithData(NULL, bitmap, 256 * 256 * 4, NULL);
	cgSpace = CGColorSpaceCreateDeviceRGB();
	cgRect = CGRectMake(0, 0, 256, 256);
	cgImage = CGImageCreate(256, 256, 8, 32, 256 * 4, cgSpace, kCGImageAlphaNoneSkipLast, cgData, NULL, 0, kCGRenderingIntentDefault);
	CGContextDrawImage(cgContext, cgRect, cgImage);
	CGDataProviderRelease(cgData);
	CGColorSpaceRelease(cgSpace);
	CGImageRelease(cgImage);
}

- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (void)keyDown:(NSEvent *)event
{
	NSString *characters = [event characters];
	unichar character = [characters characterAtIndex:0];

	switch(character)	{
		case NSUpArrowFunctionKey:
			sms_button_pressed(SMS_PAD1_UP);
			break;
		case NSDownArrowFunctionKey:
			sms_button_pressed(SMS_PAD1_DOWN);
			break;
		case NSLeftArrowFunctionKey:
			sms_button_pressed(SMS_PAD1_LEFT);
			break;
		case NSRightArrowFunctionKey:
			sms_button_pressed(SMS_PAD1_RIGHT);
			break;
		case '\'':
			sms_button_pressed(SMS_PAD1_B);
			break;
		case ';':
			sms_button_pressed(SMS_PAD1_A);
			break;
		case ' ':
			sms_button_pressed(GG_START);
			break;
		default:
			[super keyDown:event];
			break;
	}
}

- (void)keyUp:(NSEvent *)event
{
	NSString *characters = [event characters];
	unichar character = [characters characterAtIndex:0];

	switch(character)	{
		case NSUpArrowFunctionKey:
			sms_button_released(SMS_PAD1_UP);
			break;
		case NSDownArrowFunctionKey:
			sms_button_released(SMS_PAD1_DOWN);
			break;
		case NSLeftArrowFunctionKey:
			sms_button_released(SMS_PAD1_LEFT);
			break;
		case NSRightArrowFunctionKey:
			sms_button_released(SMS_PAD1_RIGHT);
			break;
		case '\'':
			sms_button_released(SMS_PAD1_B);
			break;
		case ';':
			sms_button_released(SMS_PAD1_A);
			break;
		case ' ':
			sms_button_released(GG_START);
			break;
		default:
			[super keyUp:event];
			break;
	}
}

@end
