/*
	This file is part of CrabEmu.

	Copyright (C) 2005, 2006 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "CrabZ80.h"
#include "CrabZ80_tables.h"
#include "CrabZ80_macros.h"
#include <stdio.h>
#include <assert.h>

typedef void (*opcodeptr)();

#define PROTOTYPE(prefix) \
static inline void _op_##prefix##00(); static inline void _op_##prefix##01();  \
static inline void _op_##prefix##02(); static inline void _op_##prefix##03();  \
static inline void _op_##prefix##04(); static inline void _op_##prefix##05();  \
static inline void _op_##prefix##06(); static inline void _op_##prefix##07();  \
static inline void _op_##prefix##08(); static inline void _op_##prefix##09();  \
static inline void _op_##prefix##0A(); static inline void _op_##prefix##0B();  \
static inline void _op_##prefix##0C(); static inline void _op_##prefix##0D();  \
static inline void _op_##prefix##0E(); static inline void _op_##prefix##0F();  \
static inline void _op_##prefix##10(); static inline void _op_##prefix##11();  \
static inline void _op_##prefix##12(); static inline void _op_##prefix##13();  \
static inline void _op_##prefix##14(); static inline void _op_##prefix##15();  \
static inline void _op_##prefix##16(); static inline void _op_##prefix##17();  \
static inline void _op_##prefix##18(); static inline void _op_##prefix##19();  \
static inline void _op_##prefix##1A(); static inline void _op_##prefix##1B();  \
static inline void _op_##prefix##1C(); static inline void _op_##prefix##1D();  \
static inline void _op_##prefix##1E(); static inline void _op_##prefix##1F();  \
static inline void _op_##prefix##20(); static inline void _op_##prefix##21();  \
static inline void _op_##prefix##22(); static inline void _op_##prefix##23();  \
static inline void _op_##prefix##24(); static inline void _op_##prefix##25();  \
static inline void _op_##prefix##26(); static inline void _op_##prefix##27();  \
static inline void _op_##prefix##28(); static inline void _op_##prefix##29();  \
static inline void _op_##prefix##2A(); static inline void _op_##prefix##2B();  \
static inline void _op_##prefix##2C(); static inline void _op_##prefix##2D();  \
static inline void _op_##prefix##2E(); static inline void _op_##prefix##2F();  \
static inline void _op_##prefix##30(); static inline void _op_##prefix##31();  \
static inline void _op_##prefix##32(); static inline void _op_##prefix##33();  \
static inline void _op_##prefix##34(); static inline void _op_##prefix##35();  \
static inline void _op_##prefix##36(); static inline void _op_##prefix##37();  \
static inline void _op_##prefix##38(); static inline void _op_##prefix##39();  \
static inline void _op_##prefix##3A(); static inline void _op_##prefix##3B();  \
static inline void _op_##prefix##3C(); static inline void _op_##prefix##3D();  \
static inline void _op_##prefix##3E(); static inline void _op_##prefix##3F();  \
static inline void _op_##prefix##40(); static inline void _op_##prefix##41();  \
static inline void _op_##prefix##42(); static inline void _op_##prefix##43();  \
static inline void _op_##prefix##44(); static inline void _op_##prefix##45();  \
static inline void _op_##prefix##46(); static inline void _op_##prefix##47();  \
static inline void _op_##prefix##48(); static inline void _op_##prefix##49();  \
static inline void _op_##prefix##4A(); static inline void _op_##prefix##4B();  \
static inline void _op_##prefix##4C(); static inline void _op_##prefix##4D();  \
static inline void _op_##prefix##4E(); static inline void _op_##prefix##4F();  \
static inline void _op_##prefix##50(); static inline void _op_##prefix##51();  \
static inline void _op_##prefix##52(); static inline void _op_##prefix##53();  \
static inline void _op_##prefix##54(); static inline void _op_##prefix##55();  \
static inline void _op_##prefix##56(); static inline void _op_##prefix##57();  \
static inline void _op_##prefix##58(); static inline void _op_##prefix##59();  \
static inline void _op_##prefix##5A(); static inline void _op_##prefix##5B();  \
static inline void _op_##prefix##5C(); static inline void _op_##prefix##5D();  \
static inline void _op_##prefix##5E(); static inline void _op_##prefix##5F();  \
static inline void _op_##prefix##60(); static inline void _op_##prefix##61();  \
static inline void _op_##prefix##62(); static inline void _op_##prefix##63();  \
static inline void _op_##prefix##64(); static inline void _op_##prefix##65();  \
static inline void _op_##prefix##66(); static inline void _op_##prefix##67();  \
static inline void _op_##prefix##68(); static inline void _op_##prefix##69();  \
static inline void _op_##prefix##6A(); static inline void _op_##prefix##6B();  \
static inline void _op_##prefix##6C(); static inline void _op_##prefix##6D();  \
static inline void _op_##prefix##6E(); static inline void _op_##prefix##6F();  \
static inline void _op_##prefix##70(); static inline void _op_##prefix##71();  \
static inline void _op_##prefix##72(); static inline void _op_##prefix##73();  \
static inline void _op_##prefix##74(); static inline void _op_##prefix##75();  \
static inline void _op_##prefix##76(); static inline void _op_##prefix##77();  \
static inline void _op_##prefix##78(); static inline void _op_##prefix##79();  \
static inline void _op_##prefix##7A(); static inline void _op_##prefix##7B();  \
static inline void _op_##prefix##7C(); static inline void _op_##prefix##7D();  \
static inline void _op_##prefix##7E(); static inline void _op_##prefix##7F();  \
static inline void _op_##prefix##80(); static inline void _op_##prefix##81();  \
static inline void _op_##prefix##82(); static inline void _op_##prefix##83();  \
static inline void _op_##prefix##84(); static inline void _op_##prefix##85();  \
static inline void _op_##prefix##86(); static inline void _op_##prefix##87();  \
static inline void _op_##prefix##88(); static inline void _op_##prefix##89();  \
static inline void _op_##prefix##8A(); static inline void _op_##prefix##8B();  \
static inline void _op_##prefix##8C(); static inline void _op_##prefix##8D();  \
static inline void _op_##prefix##8E(); static inline void _op_##prefix##8F();  \
static inline void _op_##prefix##90(); static inline void _op_##prefix##91();  \
static inline void _op_##prefix##92(); static inline void _op_##prefix##93();  \
static inline void _op_##prefix##94(); static inline void _op_##prefix##95();  \
static inline void _op_##prefix##96(); static inline void _op_##prefix##97();  \
static inline void _op_##prefix##98(); static inline void _op_##prefix##99();  \
static inline void _op_##prefix##9A(); static inline void _op_##prefix##9B();  \
static inline void _op_##prefix##9C(); static inline void _op_##prefix##9D();  \
static inline void _op_##prefix##9E(); static inline void _op_##prefix##9F();  \
static inline void _op_##prefix##A0(); static inline void _op_##prefix##A1();  \
static inline void _op_##prefix##A2(); static inline void _op_##prefix##A3();  \
static inline void _op_##prefix##A4(); static inline void _op_##prefix##A5();  \
static inline void _op_##prefix##A6(); static inline void _op_##prefix##A7();  \
static inline void _op_##prefix##A8(); static inline void _op_##prefix##A9();  \
static inline void _op_##prefix##AA(); static inline void _op_##prefix##AB();  \
static inline void _op_##prefix##AC(); static inline void _op_##prefix##AD();  \
static inline void _op_##prefix##AE(); static inline void _op_##prefix##AF();  \
static inline void _op_##prefix##B0(); static inline void _op_##prefix##B1();  \
static inline void _op_##prefix##B2(); static inline void _op_##prefix##B3();  \
static inline void _op_##prefix##B4(); static inline void _op_##prefix##B5();  \
static inline void _op_##prefix##B6(); static inline void _op_##prefix##B7();  \
static inline void _op_##prefix##B8(); static inline void _op_##prefix##B9();  \
static inline void _op_##prefix##BA(); static inline void _op_##prefix##BB();  \
static inline void _op_##prefix##BC(); static inline void _op_##prefix##BD();  \
static inline void _op_##prefix##BE(); static inline void _op_##prefix##BF();  \
static inline void _op_##prefix##C0(); static inline void _op_##prefix##C1();  \
static inline void _op_##prefix##C2(); static inline void _op_##prefix##C3();  \
static inline void _op_##prefix##C4(); static inline void _op_##prefix##C5();  \
static inline void _op_##prefix##C6(); static inline void _op_##prefix##C7();  \
static inline void _op_##prefix##C8(); static inline void _op_##prefix##C9();  \
static inline void _op_##prefix##CA(); static inline void _op_##prefix##CB();  \
static inline void _op_##prefix##CC(); static inline void _op_##prefix##CD();  \
static inline void _op_##prefix##CE(); static inline void _op_##prefix##CF();  \
static inline void _op_##prefix##D0(); static inline void _op_##prefix##D1();  \
static inline void _op_##prefix##D2(); static inline void _op_##prefix##D3();  \
static inline void _op_##prefix##D4(); static inline void _op_##prefix##D5();  \
static inline void _op_##prefix##D6(); static inline void _op_##prefix##D7();  \
static inline void _op_##prefix##D8(); static inline void _op_##prefix##D9();  \
static inline void _op_##prefix##DA(); static inline void _op_##prefix##DB();  \
static inline void _op_##prefix##DC(); static inline void _op_##prefix##DD();  \
static inline void _op_##prefix##DE(); static inline void _op_##prefix##DF();  \
static inline void _op_##prefix##E0(); static inline void _op_##prefix##E1();  \
static inline void _op_##prefix##E2(); static inline void _op_##prefix##E3();  \
static inline void _op_##prefix##E4(); static inline void _op_##prefix##E5();  \
static inline void _op_##prefix##E6(); static inline void _op_##prefix##E7();  \
static inline void _op_##prefix##E8(); static inline void _op_##prefix##E9();  \
static inline void _op_##prefix##EA(); static inline void _op_##prefix##EB();  \
static inline void _op_##prefix##EC(); static inline void _op_##prefix##ED();  \
static inline void _op_##prefix##EE(); static inline void _op_##prefix##EF();  \
static inline void _op_##prefix##F0(); static inline void _op_##prefix##F1();  \
static inline void _op_##prefix##F2(); static inline void _op_##prefix##F3();  \
static inline void _op_##prefix##F4(); static inline void _op_##prefix##F5();  \
static inline void _op_##prefix##F6(); static inline void _op_##prefix##F7();  \
static inline void _op_##prefix##F8(); static inline void _op_##prefix##F9();  \
static inline void _op_##prefix##FA(); static inline void _op_##prefix##FB();  \
static inline void _op_##prefix##FC(); static inline void _op_##prefix##FD();  \
static inline void _op_##prefix##FE(); static inline void _op_##prefix##FF();  \
\
const static opcodeptr _op##prefix##_tab[0x100] =  {   \
    _op_##prefix##00, _op_##prefix##01, _op_##prefix##02, _op_##prefix##03, \
    _op_##prefix##04, _op_##prefix##05, _op_##prefix##06, _op_##prefix##07, \
    _op_##prefix##08, _op_##prefix##09, _op_##prefix##0A, _op_##prefix##0B, \
    _op_##prefix##0C, _op_##prefix##0D, _op_##prefix##0E, _op_##prefix##0F, \
    _op_##prefix##10, _op_##prefix##11, _op_##prefix##12, _op_##prefix##13, \
    _op_##prefix##14, _op_##prefix##15, _op_##prefix##16, _op_##prefix##17, \
    _op_##prefix##18, _op_##prefix##19, _op_##prefix##1A, _op_##prefix##1B, \
    _op_##prefix##1C, _op_##prefix##1D, _op_##prefix##1E, _op_##prefix##1F, \
    _op_##prefix##20, _op_##prefix##21, _op_##prefix##22, _op_##prefix##23, \
    _op_##prefix##24, _op_##prefix##25, _op_##prefix##26, _op_##prefix##27, \
    _op_##prefix##28, _op_##prefix##29, _op_##prefix##2A, _op_##prefix##2B, \
    _op_##prefix##2C, _op_##prefix##2D, _op_##prefix##2E, _op_##prefix##2F, \
    _op_##prefix##30, _op_##prefix##31, _op_##prefix##32, _op_##prefix##33, \
    _op_##prefix##34, _op_##prefix##35, _op_##prefix##36, _op_##prefix##37, \
    _op_##prefix##38, _op_##prefix##39, _op_##prefix##3A, _op_##prefix##3B, \
    _op_##prefix##3C, _op_##prefix##3D, _op_##prefix##3E, _op_##prefix##3F, \
    _op_##prefix##40, _op_##prefix##41, _op_##prefix##42, _op_##prefix##43, \
    _op_##prefix##44, _op_##prefix##45, _op_##prefix##46, _op_##prefix##47, \
    _op_##prefix##48, _op_##prefix##49, _op_##prefix##4A, _op_##prefix##4B, \
    _op_##prefix##4C, _op_##prefix##4D, _op_##prefix##4E, _op_##prefix##4F, \
    _op_##prefix##50, _op_##prefix##51, _op_##prefix##52, _op_##prefix##53, \
    _op_##prefix##54, _op_##prefix##55, _op_##prefix##56, _op_##prefix##57, \
    _op_##prefix##58, _op_##prefix##59, _op_##prefix##5A, _op_##prefix##5B, \
    _op_##prefix##5C, _op_##prefix##5D, _op_##prefix##5E, _op_##prefix##5F, \
    _op_##prefix##60, _op_##prefix##61, _op_##prefix##62, _op_##prefix##63, \
    _op_##prefix##64, _op_##prefix##65, _op_##prefix##66, _op_##prefix##67, \
    _op_##prefix##68, _op_##prefix##69, _op_##prefix##6A, _op_##prefix##6B, \
    _op_##prefix##6C, _op_##prefix##6D, _op_##prefix##6E, _op_##prefix##6F, \
    _op_##prefix##70, _op_##prefix##71, _op_##prefix##72, _op_##prefix##73, \
    _op_##prefix##74, _op_##prefix##75, _op_##prefix##76, _op_##prefix##77, \
    _op_##prefix##78, _op_##prefix##79, _op_##prefix##7A, _op_##prefix##7B, \
    _op_##prefix##7C, _op_##prefix##7D, _op_##prefix##7E, _op_##prefix##7F, \
    _op_##prefix##80, _op_##prefix##81, _op_##prefix##82, _op_##prefix##83, \
    _op_##prefix##84, _op_##prefix##85, _op_##prefix##86, _op_##prefix##87, \
    _op_##prefix##88, _op_##prefix##89, _op_##prefix##8A, _op_##prefix##8B, \
    _op_##prefix##8C, _op_##prefix##8D, _op_##prefix##8E, _op_##prefix##8F, \
    _op_##prefix##90, _op_##prefix##91, _op_##prefix##92, _op_##prefix##93, \
    _op_##prefix##94, _op_##prefix##95, _op_##prefix##96, _op_##prefix##97, \
    _op_##prefix##98, _op_##prefix##99, _op_##prefix##9A, _op_##prefix##9B, \
    _op_##prefix##9C, _op_##prefix##9D, _op_##prefix##9E, _op_##prefix##9F, \
    _op_##prefix##A0, _op_##prefix##A1, _op_##prefix##A2, _op_##prefix##A3, \
    _op_##prefix##A4, _op_##prefix##A5, _op_##prefix##A6, _op_##prefix##A7, \
    _op_##prefix##A8, _op_##prefix##A9, _op_##prefix##AA, _op_##prefix##AB, \
    _op_##prefix##AC, _op_##prefix##AD, _op_##prefix##AE, _op_##prefix##AF, \
    _op_##prefix##B0, _op_##prefix##B1, _op_##prefix##B2, _op_##prefix##B3, \
    _op_##prefix##B4, _op_##prefix##B5, _op_##prefix##B6, _op_##prefix##B7, \
    _op_##prefix##B8, _op_##prefix##B9, _op_##prefix##BA, _op_##prefix##BB, \
    _op_##prefix##BC, _op_##prefix##BD, _op_##prefix##BE, _op_##prefix##BF, \
    _op_##prefix##C0, _op_##prefix##C1, _op_##prefix##C2, _op_##prefix##C3, \
    _op_##prefix##C4, _op_##prefix##C5, _op_##prefix##C6, _op_##prefix##C7, \
    _op_##prefix##C8, _op_##prefix##C9, _op_##prefix##CA, _op_##prefix##CB, \
    _op_##prefix##CC, _op_##prefix##CD, _op_##prefix##CE, _op_##prefix##CF, \
    _op_##prefix##D0, _op_##prefix##D1, _op_##prefix##D2, _op_##prefix##D3, \
    _op_##prefix##D4, _op_##prefix##D5, _op_##prefix##D6, _op_##prefix##D7, \
    _op_##prefix##D8, _op_##prefix##D9, _op_##prefix##DA, _op_##prefix##DB, \
    _op_##prefix##DC, _op_##prefix##DD, _op_##prefix##DE, _op_##prefix##DF, \
    _op_##prefix##E0, _op_##prefix##E1, _op_##prefix##E2, _op_##prefix##E3, \
    _op_##prefix##E4, _op_##prefix##E5, _op_##prefix##E6, _op_##prefix##E7, \
    _op_##prefix##E8, _op_##prefix##E9, _op_##prefix##EA, _op_##prefix##EB, \
    _op_##prefix##EC, _op_##prefix##ED, _op_##prefix##EE, _op_##prefix##EF, \
    _op_##prefix##F0, _op_##prefix##F1, _op_##prefix##F2, _op_##prefix##F3, \
    _op_##prefix##F4, _op_##prefix##F5, _op_##prefix##F6, _op_##prefix##F7, \
    _op_##prefix##F8, _op_##prefix##F9, _op_##prefix##FA, _op_##prefix##FB, \
    _op_##prefix##FC, _op_##prefix##FD, _op_##prefix##FE, _op_##prefix##FF \
};

PROTOTYPE()
PROTOTYPE(CB)
PROTOTYPE(DFD)
PROTOTYPE(ED)
PROTOTYPE(DFDCB)

static CrabZ80_t *cpu = NULL;

static uint8 CrabZ80_dummy_read(uint16 addr)	{
	return 0;
}

static void CrabZ80_dummy_write(uint16 addr, uint8 data)	{
}

void CrabZ80_set_portread(CrabZ80_t *cpu, uint8 (*pread)(uint16 port))	{
	if(pread == NULL)
		cpu->pread = CrabZ80_dummy_read;
	else
		cpu->pread = pread;
}

void CrabZ80_set_memread(CrabZ80_t *cpu, uint8 (*mread)(uint16 addr))	{
	if(mread == NULL)
		cpu->mread = CrabZ80_dummy_read;
	else
		cpu->mread = mread;
}

void CrabZ80_set_portwrite(CrabZ80_t *cpu, void (*pwrite)(uint16 port, uint8 data))	{
	if(pwrite == NULL)
		cpu->pwrite = CrabZ80_dummy_write;
	else
		cpu->pwrite = pwrite;
}

void CrabZ80_set_memwrite(CrabZ80_t *cpu, void (*mwrite)(uint16 addr, uint8 data))	{
	if(mwrite == NULL)
		cpu->mwrite = CrabZ80_dummy_write;
	else
		cpu->mwrite = mwrite;
}

void CrabZ80_init(CrabZ80_t *cpu)	{
	cpu->pread = CrabZ80_dummy_read;
	cpu->mread = CrabZ80_dummy_read;
	cpu->pwrite = CrabZ80_dummy_write;
	cpu->mwrite = CrabZ80_dummy_write;
}

void CrabZ80_reset(CrabZ80_t *cpu)	{
	cpu->pc.w = 0x0000;
	cpu->iff1 = 0;
	cpu->iff2 = 0;
	cpu->im = 0;
	cpu->halt = 0;
	cpu->r_top = 0;
	cpu->af.w = 0x0040;
	cpu->bc.w = 0x0000;
	cpu->de.w = 0x0000;
	cpu->hl.w = 0x0000;
	cpu->ix.w = 0xFFFF;
	cpu->iy.w = 0xFFFF;
	cpu->sp.w = 0x0000;
	cpu->ir.w = 0x0000;
	cpu->afp.w = 0x0000;
	cpu->bcp.w = 0x0000;
	cpu->dep.w = 0x0000;
	cpu->hlp.w = 0x0000;
	cpu->cycles = 0;
}

void CrabZ80_pulse_nmi(CrabZ80_t *cpu)	{
	if(cpu->halt)	{
		cpu->pc.w++;
		cpu->halt = 0;
	}

	cpu->iff2 = cpu->iff1;
	cpu->iff1 = 0;
	cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
	cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
	cpu->pc.w = 0x0066;
	cpu->cycles += 11;

	cpu->ir.b.l = (cpu->ir.b.l & 0x80) | ((cpu->ir.b.l + 1) & 0x7F);
}

void CrabZ80_pulse_irq(CrabZ80_t *cpu)	{
	if(cpu->iff1 == 0) /* interrupts are disabled. */
		return;

	if(cpu->halt)	{
		cpu->pc.w++;
		cpu->halt = 0;
	}

	cpu->iff1 = cpu->iff2 = 0;

	switch(cpu->im)	{
		case 1:
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = 0x0038;
			cpu->cycles += 13;
			break;
		default:
			printf("Unimplemented Interrupt Mode: %d\n", cpu->im);
			assert(0);
	}
}

#define EXECUTE(prefix, op) {   \
    uint8 _tmp = (op); \
    cpu->cycles += cycles##prefix##_tab[_tmp]; \
    _op##prefix##_tab[_tmp](); \
}

uint32 CrabZ80_execute(CrabZ80_t *cpuin, uint32 cycles)	{
	uint8 inst;
	uint32 real_cycles;

    cpu = cpuin;

	while(cpu->cycles < cycles)	{
		cpu->ei = 0;
		inst = cpu->mread(cpu->pc.w++);
        EXECUTE(, inst);
		++cpu->ir.b.l;
	}

	real_cycles = cpu->cycles;
	cpu->cycles = 0;

    cpu = NULL;

	return real_cycles;
}

#define opcode(prefix, number)  \
    static inline void _op_ ##prefix ##number ()

opcode(, 00)    { }
opcode(, 01)    { OP_LD16FPC(cpu->bc); }
opcode(, 02)    { OP_LD8TMEM(cpu->af.b.h, cpu->bc.w); }
opcode(, 03)    { OP_INC16(cpu->bc.w); }
opcode(, 04)    { OP_INC8(cpu->bc.b.h); }
opcode(, 05)    { OP_DEC8(cpu->bc.b.h); }
opcode(, 06)    { OP_LD8FMEM(cpu->bc.b.h, cpu->pc.w++); }
opcode(, 07)    { OP_RLCA(); }
opcode(, 08)    { OP_EX(cpu->af, cpu->afp); }
opcode(, 09)    { OP_ADD16(cpu->hl, cpu->bc); }
opcode(, 0A)    { OP_LD8FMEM(cpu->af.b.h, cpu->bc.w); }
opcode(, 0B)    { OP_DEC16(cpu->bc.w); }
opcode(, 0C)    { OP_INC8(cpu->bc.b.l); }
opcode(, 0D)    { OP_DEC8(cpu->bc.b.l); }
opcode(, 0E)    { OP_LD8FMEM(cpu->bc.b.l, cpu->pc.w++); }
opcode(, 0F)    { OP_RRCA(); }
opcode(, 10)    { --cpu->bc.b.h; OP_CONDJR(cpu->bc.b.h); }
opcode(, 11)    { OP_LD16FPC(cpu->de); }
opcode(, 12)    { OP_LD8TMEM(cpu->af.b.h, cpu->de.w); }
opcode(, 13)    { OP_INC16(cpu->de.w); }
opcode(, 14)    { OP_INC8(cpu->de.b.h); }
opcode(, 15)    { OP_DEC8(cpu->de.b.h); }
opcode(, 16)    { OP_LD8FMEM(cpu->de.b.h, cpu->pc.w++); }
opcode(, 17)    { OP_RLA(); }
opcode(, 18)    { OP_JR(); }
opcode(, 19)    { OP_ADD16(cpu->hl, cpu->de); }
opcode(, 1A)    { OP_LD8FMEM(cpu->af.b.h, cpu->de.w); }
opcode(, 1B)    { OP_DEC16(cpu->de.w); }
opcode(, 1C)    { OP_INC8(cpu->de.b.l); }
opcode(, 1D)    { OP_DEC8(cpu->de.b.l); }
opcode(, 1E)    { OP_LD8FMEM(cpu->de.b.l, cpu->pc.w++); }
opcode(, 1F)    { OP_RRA(); }
opcode(, 20)    { OP_CONDJR(!(cpu->af.b.l & 0x40)); }
opcode(, 21)    { OP_LD16FPC(cpu->hl); }
opcode(, 22)    { OP_LD16TPCA(cpu->hl); }
opcode(, 23)    { OP_INC16(cpu->hl.w); }
opcode(, 24)    { OP_INC8(cpu->hl.b.h); }
opcode(, 25)    { OP_DEC8(cpu->hl.b.h); }
opcode(, 26)    { OP_LD8FMEM(cpu->hl.b.h, cpu->pc.w++); }
opcode(, 27)    { OP_DAA(); }
opcode(, 28)    { OP_CONDJR(cpu->af.b.l & 0x40); }
opcode(, 29)    { OP_ADD16(cpu->hl, cpu->hl); }
opcode(, 2A)    { OP_LD16FPCA(cpu->hl); }
opcode(, 2B)    { OP_DEC16(cpu->hl.w); }
opcode(, 2C)    { OP_INC8(cpu->hl.b.l); }
opcode(, 2D)    { OP_DEC8(cpu->hl.b.l); }
opcode(, 2E)    { OP_LD8FMEM(cpu->hl.b.l, cpu->pc.w++); }
opcode(, 2F)    { OP_CPL(); }
opcode(, 30)    { OP_CONDJR(!(cpu->af.b.l & 0x01)); }
opcode(, 31)    { OP_LD16FPC(cpu->sp); }
opcode(, 32)    { OP_LD8TPCA(cpu->af.b.h); }
opcode(, 33)    { OP_INC16(cpu->sp.w); }
opcode(, 34)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_INC8(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(, 35)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_DEC8(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(, 36)    { OP_LD8TMEM(cpu->mread(cpu->pc.w++), cpu->hl.w); }
opcode(, 37)    { OP_SCF(); }
opcode(, 38)    { OP_CONDJR(cpu->af.b.l & 0x01); }
opcode(, 39)    { OP_ADD16(cpu->hl, cpu->sp); }
opcode(, 3A)    { OP_LD8FPCA(cpu->af.b.h); }
opcode(, 3B)    { OP_DEC16(cpu->sp.w); }
opcode(, 3C)    { OP_INC8(cpu->af.b.h); }
opcode(, 3D)    { OP_DEC8(cpu->af.b.h); }
opcode(, 3E)    { OP_LD8FMEM(cpu->af.b.h, cpu->pc.w++); }
opcode(, 3F)    { OP_CCF(); }
opcode(, 40)    { }
opcode(, 41)    { cpu->bc.b.h = cpu->bc.b.l; }
opcode(, 42)    { cpu->bc.b.h = cpu->de.b.h; }
opcode(, 43)    { cpu->bc.b.h = cpu->de.b.l; }
opcode(, 44)    { cpu->bc.b.h = cpu->hl.b.h; }
opcode(, 45)    { cpu->bc.b.h = cpu->hl.b.l; }
opcode(, 46)    { cpu->bc.b.h = cpu->mread(cpu->hl.w); }
opcode(, 47)    { cpu->bc.b.h = cpu->af.b.h; }
opcode(, 48)    { cpu->bc.b.l = cpu->bc.b.h; }
opcode(, 49)    { }
opcode(, 4A)    { cpu->bc.b.l = cpu->de.b.h; }
opcode(, 4B)    { cpu->bc.b.l = cpu->de.b.l; }
opcode(, 4C)    { cpu->bc.b.l = cpu->hl.b.h; }
opcode(, 4D)    { cpu->bc.b.l = cpu->hl.b.l; }
opcode(, 4E)    { cpu->bc.b.l = cpu->mread(cpu->hl.w); }
opcode(, 4F)    { cpu->bc.b.l = cpu->af.b.h; }
opcode(, 50)    { cpu->de.b.h = cpu->bc.b.h; }
opcode(, 51)    { cpu->de.b.h = cpu->bc.b.l; }
opcode(, 52)    { }
opcode(, 53)    { cpu->de.b.h = cpu->de.b.l; }
opcode(, 54)    { cpu->de.b.h = cpu->hl.b.h; }
opcode(, 55)    { cpu->de.b.h = cpu->hl.b.l; }
opcode(, 56)    { cpu->de.b.h = cpu->mread(cpu->hl.w); }
opcode(, 57)    { cpu->de.b.h = cpu->af.b.h; }
opcode(, 58)    { cpu->de.b.l = cpu->bc.b.h; }
opcode(, 59)    { cpu->de.b.l = cpu->bc.b.l; }
opcode(, 5A)    { cpu->de.b.l = cpu->de.b.h; }
opcode(, 5B)    { }
opcode(, 5C)    { cpu->de.b.l = cpu->hl.b.h; }
opcode(, 5D)    { cpu->de.b.l = cpu->hl.b.l; }
opcode(, 5E)    { cpu->de.b.l = cpu->mread(cpu->hl.w); }
opcode(, 5F)    { cpu->de.b.l = cpu->af.b.h; }
opcode(, 60)    { cpu->hl.b.h = cpu->bc.b.h; }
opcode(, 61)    { cpu->hl.b.h = cpu->bc.b.l; }
opcode(, 62)    { cpu->hl.b.h = cpu->de.b.h; }
opcode(, 63)    { cpu->hl.b.h = cpu->de.b.l; }
opcode(, 64)    { }
opcode(, 65)    { cpu->hl.b.h = cpu->hl.b.l; }
opcode(, 66)    { cpu->hl.b.h = cpu->mread(cpu->hl.w); }
opcode(, 67)    { cpu->hl.b.h = cpu->af.b.h; }
opcode(, 68)    { cpu->hl.b.l = cpu->bc.b.h; }
opcode(, 69)    { cpu->hl.b.l = cpu->bc.b.l; }
opcode(, 6A)    { cpu->hl.b.l = cpu->de.b.h; }
opcode(, 6B)    { cpu->hl.b.l = cpu->de.b.l; }
opcode(, 6C)    { cpu->hl.b.l = cpu->hl.b.h; }
opcode(, 6D)    { }
opcode(, 6E)    { cpu->hl.b.l = cpu->mread(cpu->hl.w); }
opcode(, 6F)    { cpu->hl.b.l = cpu->af.b.h; }
opcode(, 70)    { cpu->mwrite(cpu->hl.w, cpu->bc.b.h); }
opcode(, 71)    { cpu->mwrite(cpu->hl.w, cpu->bc.b.l); }
opcode(, 72)    { cpu->mwrite(cpu->hl.w, cpu->de.b.h); }
opcode(, 73)    { cpu->mwrite(cpu->hl.w, cpu->de.b.l); }
opcode(, 74)    { cpu->mwrite(cpu->hl.w, cpu->hl.b.h); }
opcode(, 75)    { cpu->mwrite(cpu->hl.w, cpu->hl.b.l); }
opcode(, 76)    { OP_HALT(); }
opcode(, 77)    { cpu->mwrite(cpu->hl.w, cpu->af.b.h); }
opcode(, 78)    { cpu->af.b.h = cpu->bc.b.h; }
opcode(, 79)    { cpu->af.b.h = cpu->bc.b.l; }
opcode(, 7A)    { cpu->af.b.h = cpu->de.b.h; }
opcode(, 7B)    { cpu->af.b.h = cpu->de.b.l; }
opcode(, 7C)    { cpu->af.b.h = cpu->hl.b.h; }
opcode(, 7D)    { cpu->af.b.h = cpu->hl.b.l; }
opcode(, 7E)    { cpu->af.b.h = cpu->mread(cpu->hl.w); }
opcode(, 7F)    { }
opcode(, 80)    { OP_ADD8(cpu->bc.b.h);           }
opcode(, 81)    { OP_ADD8(cpu->bc.b.l);           }
opcode(, 82)    { OP_ADD8(cpu->de.b.h);           }
opcode(, 83)    { OP_ADD8(cpu->de.b.l);           }
opcode(, 84)    { OP_ADD8(cpu->hl.b.h);           }
opcode(, 85)    { OP_ADD8(cpu->hl.b.l);           }
opcode(, 86)    { OP_ADD8(cpu->mread(cpu->hl.w)); }
opcode(, 87)    { OP_ADD8(cpu->af.b.h);           }
opcode(, 88)    { OP_ADC8(cpu->bc.b.h);           }
opcode(, 89)    { OP_ADC8(cpu->bc.b.l);           }
opcode(, 8A)    { OP_ADC8(cpu->de.b.h);           }
opcode(, 8B)    { OP_ADC8(cpu->de.b.l);           }
opcode(, 8C)    { OP_ADC8(cpu->hl.b.h);           }
opcode(, 8D)    { OP_ADC8(cpu->hl.b.l);           }
opcode(, 8E)    { OP_ADC8(cpu->mread(cpu->hl.w)); }
opcode(, 8F)    { OP_ADC8(cpu->af.b.h);           }
opcode(, 90)    { OP_SUB8(cpu->bc.b.h);           }
opcode(, 91)    { OP_SUB8(cpu->bc.b.l);           }
opcode(, 92)    { OP_SUB8(cpu->de.b.h);           }
opcode(, 93)    { OP_SUB8(cpu->de.b.l);           }
opcode(, 94)    { OP_SUB8(cpu->hl.b.h);           }
opcode(, 95)    { OP_SUB8(cpu->hl.b.l);           }
opcode(, 96)    { OP_SUB8(cpu->mread(cpu->hl.w)); }
opcode(, 97)    { OP_SUB8(cpu->af.b.h);           }
opcode(, 98)    { OP_SBC8(cpu->bc.b.h);           }
opcode(, 99)    { OP_SBC8(cpu->bc.b.l);           }
opcode(, 9A)    { OP_SBC8(cpu->de.b.h);           }
opcode(, 9B)    { OP_SBC8(cpu->de.b.l);           }
opcode(, 9C)    { OP_SBC8(cpu->hl.b.h);           }
opcode(, 9D)    { OP_SBC8(cpu->hl.b.l);           }
opcode(, 9E)    { OP_SBC8(cpu->mread(cpu->hl.w)); }
opcode(, 9F)    { OP_SBC8(cpu->af.b.h);           }
opcode(, A0)    { OP_AND8(cpu->bc.b.h);           }
opcode(, A1)    { OP_AND8(cpu->bc.b.l);           }
opcode(, A2)    { OP_AND8(cpu->de.b.h);           }
opcode(, A3)    { OP_AND8(cpu->de.b.l);           }
opcode(, A4)    { OP_AND8(cpu->hl.b.h);           }
opcode(, A5)    { OP_AND8(cpu->hl.b.l);           }
opcode(, A6)    { OP_AND8(cpu->mread(cpu->hl.w)); }
opcode(, A7)    { OP_AND8(cpu->af.b.h);           }
opcode(, A8)    { OP_XOR8(cpu->bc.b.h);           }
opcode(, A9)    { OP_XOR8(cpu->bc.b.l);           }
opcode(, AA)    { OP_XOR8(cpu->de.b.h);           }
opcode(, AB)    { OP_XOR8(cpu->de.b.l);           }
opcode(, AC)    { OP_XOR8(cpu->hl.b.h);           }
opcode(, AD)    { OP_XOR8(cpu->hl.b.l);           }
opcode(, AE)    { OP_XOR8(cpu->mread(cpu->hl.w)); }
opcode(, AF)    { OP_XOR8(cpu->af.b.h);           }
opcode(, B0)    { OP_OR8(cpu->bc.b.h);            }
opcode(, B1)    { OP_OR8(cpu->bc.b.l);            }
opcode(, B2)    { OP_OR8(cpu->de.b.h);            }
opcode(, B3)    { OP_OR8(cpu->de.b.l);            }
opcode(, B4)    { OP_OR8(cpu->hl.b.h);            }
opcode(, B5)    { OP_OR8(cpu->hl.b.l);            }
opcode(, B6)    { OP_OR8(cpu->mread(cpu->hl.w));  }
opcode(, B7)    { OP_OR8(cpu->af.b.h);            }
opcode(, B8)    { OP_CP8(cpu->bc.b.h);            }
opcode(, B9)    { OP_CP8(cpu->bc.b.l);            }
opcode(, BA)    { OP_CP8(cpu->de.b.h);            }
opcode(, BB)    { OP_CP8(cpu->de.b.l);            }
opcode(, BC)    { OP_CP8(cpu->hl.b.h);            }
opcode(, BD)    { OP_CP8(cpu->hl.b.l);            }
opcode(, BE)    { OP_CP8(cpu->mread(cpu->hl.w));  }
opcode(, BF)    { OP_CP8(cpu->af.b.h);            }
opcode(, C0)    { OP_CONDRET(!(cpu->af.b.l & 0x40)); }
opcode(, C1)    { OP_POP16(cpu->bc); }
opcode(, C2)    { OP_CONDJP(!(cpu->af.b.l & 0x40)); }
opcode(, C3)    { OP_JP(); }
opcode(, C4)    { OP_CONDCALL(!(cpu->af.b.l & 0x40)); }
opcode(, C5)    { OP_PUSH16(cpu->bc); }
opcode(, C6)    { OP_ADD8(cpu->mread(cpu->pc.w++)); }
opcode(, C7)    { OP_RST(0); }
opcode(, C8)    { OP_CONDRET(cpu->af.b.l & 0x40); }
opcode(, C9)    { OP_RET(); }
opcode(, CA)    { OP_CONDJP(cpu->af.b.l & 0x40); }
opcode(, CB)    { ++cpu->ir.b.l; EXECUTE(CB, cpu->mread(cpu->pc.w++)); }
opcode(, CC)    { OP_CONDCALL(cpu->af.b.l & 0x40); }
opcode(, CD)    { OP_CALL(); }
opcode(, CE)    { OP_ADC8(cpu->mread(cpu->pc.w++)); }
opcode(, CF)    { OP_RST(8); }
opcode(, D0)    { OP_CONDRET(!(cpu->af.b.l & 0x01)); }
opcode(, D1)    { OP_POP16(cpu->de); }
opcode(, D2)    { OP_CONDJP(!(cpu->af.b.l & 0x01)); }
opcode(, D3)    { OP_OUTPC(); }
opcode(, D4)    { OP_CONDCALL(!(cpu->af.b.l & 0x01)); }
opcode(, D5)    { OP_PUSH16(cpu->de); }
opcode(, D6)    { OP_SUB8(cpu->mread(cpu->pc.w++)); }
opcode(, D7)    { OP_RST(16); }
opcode(, D8)    { OP_CONDRET(cpu->af.b.l & 0x01); }
opcode(, D9)    { OP_EXX(); }
opcode(, DA)    { OP_CONDJP(cpu->af.b.l & 0x01); }
opcode(, DB)    { OP_INPC(); }
opcode(, DC)    { OP_CONDCALL(cpu->af.b.l & 0x01); }
opcode(, DD)    { cpu->ir.b.l += 2; cpu->offset = &cpu->ix; EXECUTE(DFD, cpu->mread(cpu->pc.w++)); }
opcode(, DE)    { OP_SBC8(cpu->mread(cpu->pc.w++)); }
opcode(, DF)    { OP_RST(24); }
opcode(, E0)    { OP_CONDRET(!(cpu->af.b.l & 0x04)); }
opcode(, E1)    { OP_POP16(cpu->hl); }
opcode(, E2)    { OP_CONDJP(!(cpu->af.b.l & 0x04)); }
opcode(, E3)    { OP_EXSP(cpu->hl); }
opcode(, E4)    { OP_CONDCALL(!(cpu->af.b.l & 0x04)); }
opcode(, E5)    { OP_PUSH16(cpu->hl); }
opcode(, E6)    { OP_AND8(cpu->mread(cpu->pc.w++)); }
opcode(, E7)    { OP_RST(32); }
opcode(, E8)    { OP_CONDRET(cpu->af.b.l & 0x04); }
opcode(, E9)    { cpu->pc.w = cpu->hl.w; }
opcode(, EA)    { OP_CONDJP(cpu->af.b.l & 0x04); }
opcode(, EB)    { OP_EX(cpu->de, cpu->hl); }
opcode(, EC)    { OP_CONDCALL(cpu->af.b.l & 0x04); }
opcode(, ED)    { ++cpu->ir.b.l; EXECUTE(ED, cpu->mread(cpu->pc.w++)); }
opcode(, EE)    { OP_XOR8(cpu->mread(cpu->pc.w++)); }
opcode(, EF)    { OP_RST(40); }
opcode(, F0)    { OP_CONDRET(!(cpu->af.b.l & 0x80)); }
opcode(, F1)    { OP_POP16(cpu->af); }
opcode(, F2)    { OP_CONDJP(!(cpu->af.b.l & 0x80)); }
opcode(, F3)    { cpu->iff1 = cpu->iff2 = 0; }
opcode(, F4)    { OP_CONDCALL(!(cpu->af.b.l & 0x80)); }
opcode(, F5)    { OP_PUSH16(cpu->af); }
opcode(, F6)    { OP_OR8(cpu->mread(cpu->pc.w++)); }
opcode(, F7)    { OP_RST(48); }
opcode(, F8)    { OP_CONDRET(cpu->af.b.l & 0x80); }
opcode(, F9)    { cpu->sp.w = cpu->hl.w; }
opcode(, FA)    { OP_CONDJP(cpu->af.b.l & 0x80); }
opcode(, FB)    { OP_EI(); }
opcode(, FC)    { OP_CONDCALL(cpu->af.b.l & 0x80); }
opcode(, FD)    { cpu->ir.b.l += 2; cpu->offset = &cpu->iy; EXECUTE(DFD, cpu->mread(cpu->pc.w++)); }
opcode(, FE)    { OP_CP8(cpu->mread(cpu->pc.w++)); }
opcode(, FF)    { OP_RST(56); }

/* CB prefixed opcodes */
opcode(CB, 00)    { OP_RLC(cpu->bc.b.h); }
opcode(CB, 01)    { OP_RLC(cpu->bc.b.l); }
opcode(CB, 02)    { OP_RLC(cpu->de.b.h); }
opcode(CB, 03)    { OP_RLC(cpu->de.b.l); }
opcode(CB, 04)    { OP_RLC(cpu->hl.b.h); }
opcode(CB, 05)    { OP_RLC(cpu->hl.b.l); }
opcode(CB, 06)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RLC(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 07)    { OP_RLC(cpu->af.b.h); }
opcode(CB, 08)    { OP_RRC(cpu->bc.b.h); }
opcode(CB, 09)    { OP_RRC(cpu->bc.b.l); }
opcode(CB, 0A)    { OP_RRC(cpu->de.b.h); }
opcode(CB, 0B)    { OP_RRC(cpu->de.b.l); }
opcode(CB, 0C)    { OP_RRC(cpu->hl.b.h); }
opcode(CB, 0D)    { OP_RRC(cpu->hl.b.l); }
opcode(CB, 0E)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RRC(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 0F)    { OP_RRC(cpu->af.b.h); }
opcode(CB, 10)    { OP_RL(cpu->bc.b.h); }
opcode(CB, 11)    { OP_RL(cpu->bc.b.l); }
opcode(CB, 12)    { OP_RL(cpu->de.b.h); }
opcode(CB, 13)    { OP_RL(cpu->de.b.l); }
opcode(CB, 14)    { OP_RL(cpu->hl.b.h); }
opcode(CB, 15)    { OP_RL(cpu->hl.b.l); }
opcode(CB, 16)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RL(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 17)    { OP_RL(cpu->af.b.h); }
opcode(CB, 18)    { OP_RR(cpu->bc.b.h); }
opcode(CB, 19)    { OP_RR(cpu->bc.b.l); }
opcode(CB, 1A)    { OP_RR(cpu->de.b.h); }
opcode(CB, 1B)    { OP_RR(cpu->de.b.l); }
opcode(CB, 1C)    { OP_RR(cpu->hl.b.h); }
opcode(CB, 1D)    { OP_RR(cpu->hl.b.l); }
opcode(CB, 1E)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RR(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 1F)    { OP_RR(cpu->af.b.h); }
opcode(CB, 20)    { OP_SLA(cpu->bc.b.h); }
opcode(CB, 21)    { OP_SLA(cpu->bc.b.l); }
opcode(CB, 22)    { OP_SLA(cpu->de.b.h); }
opcode(CB, 23)    { OP_SLA(cpu->de.b.l); }
opcode(CB, 24)    { OP_SLA(cpu->hl.b.h); }
opcode(CB, 25)    { OP_SLA(cpu->hl.b.l); }
opcode(CB, 26)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SLA(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 27)    { OP_SLA(cpu->af.b.h); }
opcode(CB, 28)    { OP_SRA(cpu->bc.b.h); }
opcode(CB, 29)    { OP_SRA(cpu->bc.b.l); }
opcode(CB, 2A)    { OP_SRA(cpu->de.b.h); }
opcode(CB, 2B)    { OP_SRA(cpu->de.b.l); }
opcode(CB, 2C)    { OP_SRA(cpu->hl.b.h); }
opcode(CB, 2D)    { OP_SRA(cpu->hl.b.l); }
opcode(CB, 2E)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SRA(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 2F)    { OP_SRA(cpu->af.b.h); }
opcode(CB, 30)    { OP_SLL(cpu->bc.b.h); }
opcode(CB, 31)    { OP_SLL(cpu->bc.b.l); }
opcode(CB, 32)    { OP_SLL(cpu->de.b.h); }
opcode(CB, 33)    { OP_SLL(cpu->de.b.l); }
opcode(CB, 34)    { OP_SLL(cpu->hl.b.h); }
opcode(CB, 35)    { OP_SLL(cpu->hl.b.l); }
opcode(CB, 36)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SLL(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 37)    { OP_SLL(cpu->af.b.h); }
opcode(CB, 38)    { OP_SRL(cpu->bc.b.h); }
opcode(CB, 39)    { OP_SRL(cpu->bc.b.l); }
opcode(CB, 3A)    { OP_SRL(cpu->de.b.h); }
opcode(CB, 3B)    { OP_SRL(cpu->de.b.l); }
opcode(CB, 3C)    { OP_SRL(cpu->hl.b.h); }
opcode(CB, 3D)    { OP_SRL(cpu->hl.b.l); }
opcode(CB, 3E)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SRL(tmp); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 3F)    { OP_SRL(cpu->af.b.h); }
opcode(CB, 40)    { OP_BIT(cpu->bc.b.h, 0); }
opcode(CB, 41)    { OP_BIT(cpu->bc.b.l, 0); }
opcode(CB, 42)    { OP_BIT(cpu->de.b.h, 0); }
opcode(CB, 43)    { OP_BIT(cpu->de.b.l, 0); }
opcode(CB, 44)    { OP_BIT(cpu->hl.b.h, 0); }
opcode(CB, 45)    { OP_BIT(cpu->hl.b.l, 0); }
opcode(CB, 46)    { OP_BIT(cpu->mread(cpu->hl.w), 0); }
opcode(CB, 47)    { OP_BIT(cpu->af.b.h, 0); }
opcode(CB, 48)    { OP_BIT(cpu->bc.b.h, 1); }
opcode(CB, 49)    { OP_BIT(cpu->bc.b.l, 1); }
opcode(CB, 4A)    { OP_BIT(cpu->de.b.h, 1); }
opcode(CB, 4B)    { OP_BIT(cpu->de.b.l, 1); }
opcode(CB, 4C)    { OP_BIT(cpu->hl.b.h, 1); }
opcode(CB, 4D)    { OP_BIT(cpu->hl.b.l, 1); }
opcode(CB, 4E)    { OP_BIT(cpu->mread(cpu->hl.w), 1); }
opcode(CB, 4F)    { OP_BIT(cpu->af.b.h, 1); }
opcode(CB, 50)    { OP_BIT(cpu->bc.b.h, 2); }
opcode(CB, 51)    { OP_BIT(cpu->bc.b.l, 2); }
opcode(CB, 52)    { OP_BIT(cpu->de.b.h, 2); }
opcode(CB, 53)    { OP_BIT(cpu->de.b.l, 2); }
opcode(CB, 54)    { OP_BIT(cpu->hl.b.h, 2); }
opcode(CB, 55)    { OP_BIT(cpu->hl.b.l, 2); }
opcode(CB, 56)    { OP_BIT(cpu->mread(cpu->hl.w), 2); }
opcode(CB, 57)    { OP_BIT(cpu->af.b.h, 2); }
opcode(CB, 58)    { OP_BIT(cpu->bc.b.h, 3); }
opcode(CB, 59)    { OP_BIT(cpu->bc.b.l, 3); }
opcode(CB, 5A)    { OP_BIT(cpu->de.b.h, 3); }
opcode(CB, 5B)    { OP_BIT(cpu->de.b.l, 3); }
opcode(CB, 5C)    { OP_BIT(cpu->hl.b.h, 3); }
opcode(CB, 5D)    { OP_BIT(cpu->hl.b.l, 3); }
opcode(CB, 5E)    { OP_BIT(cpu->mread(cpu->hl.w), 3); }
opcode(CB, 5F)    { OP_BIT(cpu->af.b.h, 3); }
opcode(CB, 60)    { OP_BIT(cpu->bc.b.h, 4); }
opcode(CB, 61)    { OP_BIT(cpu->bc.b.l, 4); }
opcode(CB, 62)    { OP_BIT(cpu->de.b.h, 4); }
opcode(CB, 63)    { OP_BIT(cpu->de.b.l, 4); }
opcode(CB, 64)    { OP_BIT(cpu->hl.b.h, 4); }
opcode(CB, 65)    { OP_BIT(cpu->hl.b.l, 4); }
opcode(CB, 66)    { OP_BIT(cpu->mread(cpu->hl.w), 4); }
opcode(CB, 67)    { OP_BIT(cpu->af.b.h, 4); }
opcode(CB, 68)    { OP_BIT(cpu->bc.b.h, 5); }
opcode(CB, 69)    { OP_BIT(cpu->bc.b.l, 5); }
opcode(CB, 6A)    { OP_BIT(cpu->de.b.h, 5); }
opcode(CB, 6B)    { OP_BIT(cpu->de.b.l, 5); }
opcode(CB, 6C)    { OP_BIT(cpu->hl.b.h, 5); }
opcode(CB, 6D)    { OP_BIT(cpu->hl.b.l, 5); }
opcode(CB, 6E)    { OP_BIT(cpu->mread(cpu->hl.w), 5); }
opcode(CB, 6F)    { OP_BIT(cpu->af.b.h, 5); }
opcode(CB, 70)    { OP_BIT(cpu->bc.b.h, 6); }
opcode(CB, 71)    { OP_BIT(cpu->bc.b.l, 6); }
opcode(CB, 72)    { OP_BIT(cpu->de.b.h, 6); }
opcode(CB, 73)    { OP_BIT(cpu->de.b.l, 6); }
opcode(CB, 74)    { OP_BIT(cpu->hl.b.h, 6); }
opcode(CB, 75)    { OP_BIT(cpu->hl.b.l, 6); }
opcode(CB, 76)    { OP_BIT(cpu->mread(cpu->hl.w), 6); }
opcode(CB, 77)    { OP_BIT(cpu->af.b.h, 6); }
opcode(CB, 78)    { OP_BIT(cpu->bc.b.h, 7); }
opcode(CB, 79)    { OP_BIT(cpu->bc.b.l, 7); }
opcode(CB, 7A)    { OP_BIT(cpu->de.b.h, 7); }
opcode(CB, 7B)    { OP_BIT(cpu->de.b.l, 7); }
opcode(CB, 7C)    { OP_BIT(cpu->hl.b.h, 7); }
opcode(CB, 7D)    { OP_BIT(cpu->hl.b.l, 7); }
opcode(CB, 7E)    { OP_BIT(cpu->mread(cpu->hl.w), 7); }
opcode(CB, 7F)    { OP_BIT(cpu->af.b.h, 7); }
opcode(CB, 80)    { OP_RES(cpu->bc.b.h, 0); }
opcode(CB, 81)    { OP_RES(cpu->bc.b.l, 0); }
opcode(CB, 82)    { OP_RES(cpu->de.b.h, 0); }
opcode(CB, 83)    { OP_RES(cpu->de.b.l, 0); }
opcode(CB, 84)    { OP_RES(cpu->hl.b.h, 0); }
opcode(CB, 85)    { OP_RES(cpu->hl.b.l, 0); }
opcode(CB, 86)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RES(tmp, 0); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 87)    { OP_RES(cpu->af.b.h, 0); }
opcode(CB, 88)    { OP_RES(cpu->bc.b.h, 1); }
opcode(CB, 89)    { OP_RES(cpu->bc.b.l, 1); }
opcode(CB, 8A)    { OP_RES(cpu->de.b.h, 1); }
opcode(CB, 8B)    { OP_RES(cpu->de.b.l, 1); }
opcode(CB, 8C)    { OP_RES(cpu->hl.b.h, 1); }
opcode(CB, 8D)    { OP_RES(cpu->hl.b.l, 1); }
opcode(CB, 8E)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RES(tmp, 1); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 8F)    { OP_RES(cpu->af.b.h, 1); }
opcode(CB, 90)    { OP_RES(cpu->bc.b.h, 2); }
opcode(CB, 91)    { OP_RES(cpu->bc.b.l, 2); }
opcode(CB, 92)    { OP_RES(cpu->de.b.h, 2); }
opcode(CB, 93)    { OP_RES(cpu->de.b.l, 2); }
opcode(CB, 94)    { OP_RES(cpu->hl.b.h, 2); }
opcode(CB, 95)    { OP_RES(cpu->hl.b.l, 2); }
opcode(CB, 96)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RES(tmp, 2); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 97)    { OP_RES(cpu->af.b.h, 2); }
opcode(CB, 98)    { OP_RES(cpu->bc.b.h, 3); }
opcode(CB, 99)    { OP_RES(cpu->bc.b.l, 3); }
opcode(CB, 9A)    { OP_RES(cpu->de.b.h, 3); }
opcode(CB, 9B)    { OP_RES(cpu->de.b.l, 3); }
opcode(CB, 9C)    { OP_RES(cpu->hl.b.h, 3); }
opcode(CB, 9D)    { OP_RES(cpu->hl.b.l, 3); }
opcode(CB, 9E)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RES(tmp, 3); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, 9F)    { OP_RES(cpu->af.b.h, 3); }
opcode(CB, A0)    { OP_RES(cpu->bc.b.h, 4); }
opcode(CB, A1)    { OP_RES(cpu->bc.b.l, 4); }
opcode(CB, A2)    { OP_RES(cpu->de.b.h, 4); }
opcode(CB, A3)    { OP_RES(cpu->de.b.l, 4); }
opcode(CB, A4)    { OP_RES(cpu->hl.b.h, 4); }
opcode(CB, A5)    { OP_RES(cpu->hl.b.l, 4); }
opcode(CB, A6)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RES(tmp, 4); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, A7)    { OP_RES(cpu->af.b.h, 4); }
opcode(CB, A8)    { OP_RES(cpu->bc.b.h, 5); }
opcode(CB, A9)    { OP_RES(cpu->bc.b.l, 5); }
opcode(CB, AA)    { OP_RES(cpu->de.b.h, 5); }
opcode(CB, AB)    { OP_RES(cpu->de.b.l, 5); }
opcode(CB, AC)    { OP_RES(cpu->hl.b.h, 5); }
opcode(CB, AD)    { OP_RES(cpu->hl.b.l, 5); }
opcode(CB, AE)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RES(tmp, 5); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, AF)    { OP_RES(cpu->af.b.h, 5); }
opcode(CB, B0)    { OP_RES(cpu->bc.b.h, 6); }
opcode(CB, B1)    { OP_RES(cpu->bc.b.l, 6); }
opcode(CB, B2)    { OP_RES(cpu->de.b.h, 6); }
opcode(CB, B3)    { OP_RES(cpu->de.b.l, 6); }
opcode(CB, B4)    { OP_RES(cpu->hl.b.h, 6); }
opcode(CB, B5)    { OP_RES(cpu->hl.b.l, 6); }
opcode(CB, B6)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RES(tmp, 6); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, B7)    { OP_RES(cpu->af.b.h, 6); }
opcode(CB, B8)    { OP_RES(cpu->bc.b.h, 7); }
opcode(CB, B9)    { OP_RES(cpu->bc.b.l, 7); }
opcode(CB, BA)    { OP_RES(cpu->de.b.h, 7); }
opcode(CB, BB)    { OP_RES(cpu->de.b.l, 7); }
opcode(CB, BC)    { OP_RES(cpu->hl.b.h, 7); }
opcode(CB, BD)    { OP_RES(cpu->hl.b.l, 7); }
opcode(CB, BE)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_RES(tmp, 7); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, BF)    { OP_RES(cpu->af.b.h, 7); }
opcode(CB, C0)    { OP_SET(cpu->bc.b.h, 0); }
opcode(CB, C1)    { OP_SET(cpu->bc.b.l, 0); }
opcode(CB, C2)    { OP_SET(cpu->de.b.h, 0); }
opcode(CB, C3)    { OP_SET(cpu->de.b.l, 0); }
opcode(CB, C4)    { OP_SET(cpu->hl.b.h, 0); }
opcode(CB, C5)    { OP_SET(cpu->hl.b.l, 0); }
opcode(CB, C6)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SET(tmp, 0); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, C7)    { OP_SET(cpu->af.b.h, 0); }
opcode(CB, C8)    { OP_SET(cpu->bc.b.h, 1); }
opcode(CB, C9)    { OP_SET(cpu->bc.b.l, 1); }
opcode(CB, CA)    { OP_SET(cpu->de.b.h, 1); }
opcode(CB, CB)    { OP_SET(cpu->de.b.l, 1); }
opcode(CB, CC)    { OP_SET(cpu->hl.b.h, 1); }
opcode(CB, CD)    { OP_SET(cpu->hl.b.l, 1); }
opcode(CB, CE)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SET(tmp, 1); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, CF)    { OP_SET(cpu->af.b.h, 1); }
opcode(CB, D0)    { OP_SET(cpu->bc.b.h, 2); }
opcode(CB, D1)    { OP_SET(cpu->bc.b.l, 2); }
opcode(CB, D2)    { OP_SET(cpu->de.b.h, 2); }
opcode(CB, D3)    { OP_SET(cpu->de.b.l, 2); }
opcode(CB, D4)    { OP_SET(cpu->hl.b.h, 2); }
opcode(CB, D5)    { OP_SET(cpu->hl.b.l, 2); }
opcode(CB, D6)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SET(tmp, 2); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, D7)    { OP_SET(cpu->af.b.h, 2); }
opcode(CB, D8)    { OP_SET(cpu->bc.b.h, 3); }
opcode(CB, D9)    { OP_SET(cpu->bc.b.l, 3); }
opcode(CB, DA)    { OP_SET(cpu->de.b.h, 3); }
opcode(CB, DB)    { OP_SET(cpu->de.b.l, 3); }
opcode(CB, DC)    { OP_SET(cpu->hl.b.h, 3); }
opcode(CB, DD)    { OP_SET(cpu->hl.b.l, 3); }
opcode(CB, DE)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SET(tmp, 3); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, DF)    { OP_SET(cpu->af.b.h, 3); }
opcode(CB, E0)    { OP_SET(cpu->bc.b.h, 4); }
opcode(CB, E1)    { OP_SET(cpu->bc.b.l, 4); }
opcode(CB, E2)    { OP_SET(cpu->de.b.h, 4); }
opcode(CB, E3)    { OP_SET(cpu->de.b.l, 4); }
opcode(CB, E4)    { OP_SET(cpu->hl.b.h, 4); }
opcode(CB, E5)    { OP_SET(cpu->hl.b.l, 4); }
opcode(CB, E6)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SET(tmp, 4); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, E7)    { OP_SET(cpu->af.b.h, 4); }
opcode(CB, E8)    { OP_SET(cpu->bc.b.h, 5); }
opcode(CB, E9)    { OP_SET(cpu->bc.b.l, 5); }
opcode(CB, EA)    { OP_SET(cpu->de.b.h, 5); }
opcode(CB, EB)    { OP_SET(cpu->de.b.l, 5); }
opcode(CB, EC)    { OP_SET(cpu->hl.b.h, 5); }
opcode(CB, ED)    { OP_SET(cpu->hl.b.l, 5); }
opcode(CB, EE)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SET(tmp, 5); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, EF)    { OP_SET(cpu->af.b.h, 5); }
opcode(CB, F0)    { OP_SET(cpu->bc.b.h, 6); }
opcode(CB, F1)    { OP_SET(cpu->bc.b.l, 6); }
opcode(CB, F2)    { OP_SET(cpu->de.b.h, 6); }
opcode(CB, F3)    { OP_SET(cpu->de.b.l, 6); }
opcode(CB, F4)    { OP_SET(cpu->hl.b.h, 6); }
opcode(CB, F5)    { OP_SET(cpu->hl.b.l, 6); }
opcode(CB, F6)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SET(tmp, 6); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, F7)    { OP_SET(cpu->af.b.h, 6); }
opcode(CB, F8)    { OP_SET(cpu->bc.b.h, 7); }
opcode(CB, F9)    { OP_SET(cpu->bc.b.l, 7); }
opcode(CB, FA)    { OP_SET(cpu->de.b.h, 7); }
opcode(CB, FB)    { OP_SET(cpu->de.b.l, 7); }
opcode(CB, FC)    { OP_SET(cpu->hl.b.h, 7); }
opcode(CB, FD)    { OP_SET(cpu->hl.b.l, 7); }
opcode(CB, FE)    { uint8 tmp = cpu->mread(cpu->hl.w); OP_SET(tmp, 7); cpu->mwrite(cpu->hl.w, tmp); }
opcode(CB, FF)    { OP_SET(cpu->af.b.h, 7); }

#define rbasei()   cpu->mread(((int8)cpu->mread(cpu->pc.w++)) + cpu->offset->w)
#define rbase()    cpu->mread(((int8)cpu->mread(cpu->pc.w)) + cpu->offset->w) 
#define wbase(val) cpu->mwrite(((int8)cpu->mread(cpu->pc.w++)) + cpu->offset->w, val)

/* DD/FD prefixed opcodes */
opcode(DFD, 00)    { _op_00(cpu); }
opcode(DFD, 01)    { _op_01(cpu); }
opcode(DFD, 02)    { _op_02(cpu); }
opcode(DFD, 03)    { _op_03(cpu); }
opcode(DFD, 04)    { _op_04(cpu); }
opcode(DFD, 05)    { _op_05(cpu); }
opcode(DFD, 06)    { _op_06(cpu); }
opcode(DFD, 07)    { _op_07(cpu); }
opcode(DFD, 08)    { _op_08(cpu); }
opcode(DFD, 09)    { OP_ADD16(*cpu->offset, cpu->bc); }
opcode(DFD, 0A)    { _op_0A(cpu); }
opcode(DFD, 0B)    { _op_0B(cpu); }
opcode(DFD, 0C)    { _op_0C(cpu); }
opcode(DFD, 0D)    { _op_0D(cpu); }
opcode(DFD, 0E)    { _op_0E(cpu); }
opcode(DFD, 0F)    { _op_0F(cpu); }
opcode(DFD, 10)    { _op_10(cpu); }
opcode(DFD, 11)    { _op_11(cpu); }
opcode(DFD, 12)    { _op_12(cpu); }
opcode(DFD, 13)    { _op_13(cpu); }
opcode(DFD, 14)    { _op_14(cpu); }
opcode(DFD, 15)    { _op_15(cpu); }
opcode(DFD, 16)    { _op_16(cpu); }
opcode(DFD, 17)    { _op_17(cpu); }
opcode(DFD, 18)    { _op_16(cpu); }
opcode(DFD, 19)    { OP_ADD16(*cpu->offset, cpu->de); }
opcode(DFD, 1A)    { _op_1A(cpu); }
opcode(DFD, 1B)    { _op_1B(cpu); }
opcode(DFD, 1C)    { _op_1C(cpu); }
opcode(DFD, 1D)    { _op_1D(cpu); }
opcode(DFD, 1E)    { _op_1E(cpu); }
opcode(DFD, 1F)    { _op_1F(cpu); }
opcode(DFD, 20)    { _op_20(cpu); }
opcode(DFD, 21)    { OP_LD16FPC(*cpu->offset); }
opcode(DFD, 22)    { OP_LD16TPCA(*cpu->offset); }
opcode(DFD, 23)    { OP_INC16(cpu->offset->w); }
opcode(DFD, 24)    { OP_INC8(cpu->offset->b.h); }
opcode(DFD, 25)    { OP_DEC8(cpu->offset->b.h); }
opcode(DFD, 26)    { OP_LD8FMEM(cpu->offset->b.h, cpu->pc.w++); }
opcode(DFD, 27)    { _op_27(cpu); }
opcode(DFD, 28)    { _op_28(cpu); }
opcode(DFD, 29)    { OP_ADD16(*cpu->offset, *cpu->offset); }
opcode(DFD, 2A)    { OP_LD16FPCA(*cpu->offset); }
opcode(DFD, 2B)    { OP_DEC16(cpu->offset->w); }
opcode(DFD, 2C)    { OP_INC8(cpu->offset->b.l); }
opcode(DFD, 2D)    { OP_DEC8(cpu->offset->b.l); }
opcode(DFD, 2E)    { OP_LD8FMEM(cpu->offset->b.l, cpu->pc.w++); }
opcode(DFD, 2F)    { _op_2F(cpu); }
opcode(DFD, 30)    { _op_30(cpu); }
opcode(DFD, 31)    { _op_31(cpu); }
opcode(DFD, 32)    { _op_32(cpu); }
opcode(DFD, 33)    { _op_33(cpu); }
opcode(DFD, 34)    { uint8 tmp = rbase(); OP_INC8(tmp); wbase(tmp); }
opcode(DFD, 35)    { uint8 tmp = rbase(); OP_DEC8(tmp); wbase(tmp); }
opcode(DFD, 36)    { OP_LD8MTM(); }
opcode(DFD, 37)    { _op_37(cpu); }
opcode(DFD, 38)    { _op_38(cpu); }
opcode(DFD, 39)    { OP_ADD16(*cpu->offset, cpu->sp); }
opcode(DFD, 3A)    { _op_3A(cpu); }
opcode(DFD, 3B)    { _op_3B(cpu); }
opcode(DFD, 3C)    { _op_3C(cpu); }
opcode(DFD, 3D)    { _op_3D(cpu); }
opcode(DFD, 3E)    { _op_3E(cpu); }
opcode(DFD, 3F)    { _op_3F(cpu); }
opcode(DFD, 40)    { _op_40(cpu); }
opcode(DFD, 41)    { _op_41(cpu); }
opcode(DFD, 42)    { _op_42(cpu); }
opcode(DFD, 43)    { _op_43(cpu); }
opcode(DFD, 44)    { cpu->bc.b.h = cpu->offset->b.h; }
opcode(DFD, 45)    { cpu->bc.b.h = cpu->offset->b.l; }
opcode(DFD, 46)    { cpu->bc.b.h = rbasei(); }
opcode(DFD, 47)    { _op_47(cpu); }
opcode(DFD, 48)    { _op_48(cpu); }
opcode(DFD, 49)    { _op_49(cpu); }
opcode(DFD, 4A)    { _op_4A(cpu); }
opcode(DFD, 4B)    { _op_4B(cpu); }
opcode(DFD, 4C)    { cpu->bc.b.l = cpu->offset->b.h; }
opcode(DFD, 4D)    { cpu->bc.b.l = cpu->offset->b.l; }
opcode(DFD, 4E)    { cpu->bc.b.l = rbasei(); }
opcode(DFD, 4F)    { _op_4F(cpu); }
opcode(DFD, 50)    { _op_50(cpu); }
opcode(DFD, 51)    { _op_51(cpu); }
opcode(DFD, 52)    { _op_52(cpu); }
opcode(DFD, 53)    { _op_53(cpu); }
opcode(DFD, 54)    { cpu->de.b.h = cpu->offset->b.h; }
opcode(DFD, 55)    { cpu->de.b.h = cpu->offset->b.l; }
opcode(DFD, 56)    { cpu->de.b.h = rbasei(); }
opcode(DFD, 57)    { _op_57(cpu); }
opcode(DFD, 58)    { _op_58(cpu); }
opcode(DFD, 59)    { _op_59(cpu); }
opcode(DFD, 5A)    { _op_5A(cpu); }
opcode(DFD, 5B)    { _op_5B(cpu); }
opcode(DFD, 5C)    { cpu->de.b.l = cpu->offset->b.h; }
opcode(DFD, 5D)    { cpu->de.b.l = cpu->offset->b.l; }
opcode(DFD, 5E)    { cpu->de.b.l = rbasei(); }
opcode(DFD, 5F)    { _op_5F(cpu); }
opcode(DFD, 60)    { cpu->offset->b.h = cpu->bc.b.h; }
opcode(DFD, 61)    { cpu->offset->b.h = cpu->bc.b.l; }
opcode(DFD, 62)    { cpu->offset->b.h = cpu->de.b.h; }
opcode(DFD, 63)    { cpu->offset->b.h = cpu->de.b.l; }
opcode(DFD, 64)    { }
opcode(DFD, 65)    { cpu->offset->b.h = cpu->offset->b.l; }
opcode(DFD, 66)    { cpu->hl.b.h = rbasei(); }
opcode(DFD, 67)    { cpu->offset->b.h = cpu->af.b.h; }
opcode(DFD, 68)    { cpu->offset->b.l = cpu->bc.b.h; }
opcode(DFD, 69)    { cpu->offset->b.l = cpu->bc.b.l; }
opcode(DFD, 6A)    { cpu->offset->b.l = cpu->de.b.h; }
opcode(DFD, 6B)    { cpu->offset->b.l = cpu->de.b.l; }
opcode(DFD, 6C)    { cpu->offset->b.l = cpu->offset->b.h; }
opcode(DFD, 6D)    { }
opcode(DFD, 6E)    { cpu->hl.b.l = rbasei(); }
opcode(DFD, 6F)    { cpu->offset->b.l = cpu->af.b.h; }
opcode(DFD, 70)    { wbase(cpu->bc.b.h); }
opcode(DFD, 71)    { wbase(cpu->bc.b.l); }
opcode(DFD, 72)    { wbase(cpu->de.b.h); }
opcode(DFD, 73)    { wbase(cpu->de.b.l); }
opcode(DFD, 74)    { wbase(cpu->hl.b.h); }
opcode(DFD, 75)    { wbase(cpu->hl.b.l); }
opcode(DFD, 76)    { _op_76(cpu); }
opcode(DFD, 77)    { wbase(cpu->af.b.h); }
opcode(DFD, 78)    { _op_78(cpu); }
opcode(DFD, 79)    { _op_79(cpu); }
opcode(DFD, 7A)    { _op_7A(cpu); }
opcode(DFD, 7B)    { _op_7B(cpu); }
opcode(DFD, 7C)    { cpu->af.b.h = cpu->offset->b.h; }
opcode(DFD, 7D)    { cpu->af.b.h = cpu->offset->b.l; }
opcode(DFD, 7E)    { cpu->af.b.h = rbasei(); }
opcode(DFD, 7F)    { _op_7F(cpu); }
opcode(DFD, 80)    { _op_80(cpu); }
opcode(DFD, 81)    { _op_81(cpu); }
opcode(DFD, 82)    { _op_82(cpu); }
opcode(DFD, 83)    { _op_83(cpu); }
opcode(DFD, 84)    { OP_ADD8(cpu->offset->b.h); }
opcode(DFD, 85)    { OP_ADD8(cpu->offset->b.l); }
opcode(DFD, 86)    { OP_ADD8(rbasei()); }
opcode(DFD, 87)    { _op_87(cpu); }
opcode(DFD, 88)    { _op_88(cpu); }
opcode(DFD, 89)    { _op_89(cpu); }
opcode(DFD, 8A)    { _op_8A(cpu); }
opcode(DFD, 8B)    { _op_8B(cpu); }
opcode(DFD, 8C)    { OP_ADC8(cpu->offset->b.h); }
opcode(DFD, 8D)    { OP_ADC8(cpu->offset->b.l); }
opcode(DFD, 8E)    { OP_ADC8(rbasei()); }
opcode(DFD, 8F)    { _op_8F(cpu); }
opcode(DFD, 90)    { _op_90(cpu); }
opcode(DFD, 91)    { _op_91(cpu); }
opcode(DFD, 92)    { _op_92(cpu); }
opcode(DFD, 93)    { _op_93(cpu); }
opcode(DFD, 94)    { OP_SUB8(cpu->offset->b.h); }
opcode(DFD, 95)    { OP_SUB8(cpu->offset->b.l); }
opcode(DFD, 96)    { OP_SUB8(rbasei()); }
opcode(DFD, 97)    { _op_97(cpu); }
opcode(DFD, 98)    { _op_98(cpu); }
opcode(DFD, 99)    { _op_99(cpu); }
opcode(DFD, 9A)    { _op_9A(cpu); }
opcode(DFD, 9B)    { _op_9B(cpu); }
opcode(DFD, 9C)    { OP_SBC8(cpu->offset->b.h); }
opcode(DFD, 9D)    { OP_SBC8(cpu->offset->b.l); }
opcode(DFD, 9E)    { OP_SBC8(rbasei()); }
opcode(DFD, 9F)    { _op_9F(cpu); }
opcode(DFD, A0)    { _op_A0(cpu); }
opcode(DFD, A1)    { _op_A1(cpu); }
opcode(DFD, A2)    { _op_A2(cpu); }
opcode(DFD, A3)    { _op_A3(cpu); }
opcode(DFD, A4)    { OP_AND8(cpu->offset->b.h); }
opcode(DFD, A5)    { OP_AND8(cpu->offset->b.l); }
opcode(DFD, A6)    { OP_AND8(rbasei()); }
opcode(DFD, A7)    { _op_A7(cpu); }
opcode(DFD, A8)    { _op_A8(cpu); }
opcode(DFD, A9)    { _op_A9(cpu); }
opcode(DFD, AA)    { _op_AA(cpu); }
opcode(DFD, AB)    { _op_AB(cpu); }
opcode(DFD, AC)    { OP_XOR8(cpu->offset->b.h); }
opcode(DFD, AD)    { OP_XOR8(cpu->offset->b.l); }
opcode(DFD, AE)    { OP_XOR8(rbasei()); }
opcode(DFD, AF)    { _op_AF(cpu); }
opcode(DFD, B0)    { _op_B0(cpu); }
opcode(DFD, B1)    { _op_B1(cpu); }
opcode(DFD, B2)    { _op_B2(cpu); }
opcode(DFD, B3)    { _op_B3(cpu); }
opcode(DFD, B4)    { OP_OR8(cpu->offset->b.h); }
opcode(DFD, B5)    { OP_OR8(cpu->offset->b.l); }
opcode(DFD, B6)    { OP_OR8(rbasei()); }
opcode(DFD, B7)    { _op_B7(cpu); }
opcode(DFD, B8)    { _op_B8(cpu); }
opcode(DFD, B9)    { _op_B9(cpu); }
opcode(DFD, BA)    { _op_BA(cpu); }
opcode(DFD, BB)    { _op_BB(cpu); }
opcode(DFD, BC)    { OP_CP8(cpu->offset->b.h); }
opcode(DFD, BD)    { OP_CP8(cpu->offset->b.l); }
opcode(DFD, BE)    { OP_CP8(rbasei()); }
opcode(DFD, BF)    { _op_BF(cpu); }
opcode(DFD, C0)    { _op_C0(cpu); }
opcode(DFD, C1)    { _op_C1(cpu); }
opcode(DFD, C2)    { _op_C2(cpu); }
opcode(DFD, C3)    { _op_C3(cpu); }
opcode(DFD, C4)    { _op_C4(cpu); }
opcode(DFD, C5)    { _op_C5(cpu); }
opcode(DFD, C6)    { _op_C6(cpu); }
opcode(DFD, C7)    { _op_C7(cpu); }
opcode(DFD, C8)    { _op_C8(cpu); }
opcode(DFD, C9)    { _op_C9(cpu); }
opcode(DFD, CA)    { _op_CA(cpu); }
opcode(DFD, CB)    { EXECUTE(DFDCB, cpu->mread(cpu->pc.w + 1)); cpu->pc.w++; }
opcode(DFD, CC)    { _op_CC(cpu); }
opcode(DFD, CD)    { _op_CD(cpu); }
opcode(DFD, CE)    { _op_CE(cpu); }
opcode(DFD, CF)    { _op_CF(cpu); }
opcode(DFD, D0)    { _op_D0(cpu); }
opcode(DFD, D1)    { _op_D1(cpu); }
opcode(DFD, D2)    { _op_D2(cpu); }
opcode(DFD, D3)    { _op_D3(cpu); }
opcode(DFD, D4)    { _op_D4(cpu); }
opcode(DFD, D5)    { _op_D5(cpu); }
opcode(DFD, D6)    { _op_D6(cpu); }
opcode(DFD, D7)    { _op_D7(cpu); }
opcode(DFD, D8)    { _op_D8(cpu); }
opcode(DFD, D9)    { _op_D9(cpu); }
opcode(DFD, DA)    { _op_DA(cpu); }
opcode(DFD, DB)    { _op_DB(cpu); }
opcode(DFD, DC)    { _op_DC(cpu); }
opcode(DFD, DD)    { _op_DD(cpu); }
opcode(DFD, DE)    { _op_DE(cpu); }
opcode(DFD, DF)    { _op_DF(cpu); }
opcode(DFD, E0)    { _op_E0(cpu); }
opcode(DFD, E1)    { OP_POP16(*cpu->offset); }
opcode(DFD, E2)    { _op_E2(cpu); }
opcode(DFD, E3)    { OP_EXSP(*cpu->offset); }
opcode(DFD, E4)    { _op_E4(cpu); }
opcode(DFD, E5)    { OP_PUSH16(*cpu->offset) }
opcode(DFD, E6)    { _op_E6(cpu); }
opcode(DFD, E7)    { _op_E7(cpu); }
opcode(DFD, E8)    { _op_E8(cpu); }
opcode(DFD, E9)    { cpu->pc.w = cpu->offset->w; }
opcode(DFD, EA)    { _op_EA(cpu); }
opcode(DFD, EB)    { _op_EB(cpu); }
opcode(DFD, EC)    { _op_EC(cpu); }
opcode(DFD, ED)    { _op_ED(cpu); }
opcode(DFD, EE)    { _op_EE(cpu); }
opcode(DFD, EF)    { _op_EF(cpu); }
opcode(DFD, F0)    { _op_F0(cpu); }
opcode(DFD, F1)    { _op_F1(cpu); }
opcode(DFD, F2)    { _op_F2(cpu); }
opcode(DFD, F3)    { _op_F3(cpu); }
opcode(DFD, F4)    { _op_F4(cpu); }
opcode(DFD, F5)    { _op_F5(cpu); }
opcode(DFD, F6)    { _op_F6(cpu); }
opcode(DFD, F7)    { _op_F7(cpu); }
opcode(DFD, F8)    { _op_F8(cpu); }
opcode(DFD, F9)    { cpu->sp.w = cpu->offset->w; }
opcode(DFD, FA)    { _op_FA(cpu); }
opcode(DFD, FB)    { _op_FB(cpu); }
opcode(DFD, FC)    { _op_FC(cpu); }
opcode(DFD, FD)    { _op_FD(cpu); }
opcode(DFD, FE)    { _op_FE(cpu); }
opcode(DFD, FF)    { _op_FF(cpu); }

/* ED prefixed opcodes */
opcode(ED, 00)    { }
opcode(ED, 01)    { }
opcode(ED, 02)    { }
opcode(ED, 03)    { }
opcode(ED, 04)    { }
opcode(ED, 05)    { }
opcode(ED, 06)    { }
opcode(ED, 07)    { }
opcode(ED, 08)    { }
opcode(ED, 09)    { }
opcode(ED, 0A)    { }
opcode(ED, 0B)    { }
opcode(ED, 0C)    { }
opcode(ED, 0D)    { }
opcode(ED, 0E)    { }
opcode(ED, 0F)    { }
opcode(ED, 10)    { }
opcode(ED, 11)    { }
opcode(ED, 12)    { }
opcode(ED, 13)    { }
opcode(ED, 14)    { }
opcode(ED, 15)    { }
opcode(ED, 16)    { }
opcode(ED, 17)    { }
opcode(ED, 18)    { }
opcode(ED, 19)    { }
opcode(ED, 1A)    { }
opcode(ED, 1B)    { }
opcode(ED, 1C)    { }
opcode(ED, 1D)    { }
opcode(ED, 1E)    { }
opcode(ED, 1F)    { }
opcode(ED, 20)    { }
opcode(ED, 21)    { }
opcode(ED, 22)    { }
opcode(ED, 23)    { }
opcode(ED, 24)    { }
opcode(ED, 25)    { }
opcode(ED, 26)    { }
opcode(ED, 27)    { }
opcode(ED, 28)    { }
opcode(ED, 29)    { }
opcode(ED, 2A)    { }
opcode(ED, 2B)    { }
opcode(ED, 2C)    { }
opcode(ED, 2D)    { }
opcode(ED, 2E)    { }
opcode(ED, 2F)    { }
opcode(ED, 30)    { }
opcode(ED, 31)    { }
opcode(ED, 32)    { }
opcode(ED, 33)    { }
opcode(ED, 34)    { }
opcode(ED, 35)    { }
opcode(ED, 36)    { }
opcode(ED, 37)    { }
opcode(ED, 38)    { }
opcode(ED, 39)    { }
opcode(ED, 3A)    { }
opcode(ED, 3B)    { }
opcode(ED, 3C)    { }
opcode(ED, 3D)    { }
opcode(ED, 3E)    { }
opcode(ED, 3F)    { }
opcode(ED, 40)    { OP_INC(cpu->bc.b.h); }
opcode(ED, 41)    { OP_OUTC(cpu->bc.b.h); }
opcode(ED, 42)    { OP_SBC16(cpu->hl.w, cpu->bc.w); }
opcode(ED, 43)    { OP_LD16TPCA(cpu->bc); }
opcode(ED, 44)    { OP_NEG(); }
opcode(ED, 45)    { OP_RETN(); }
opcode(ED, 46)    { cpu->im = 0; }
opcode(ED, 47)    { cpu->ir.b.h = cpu->af.b.h; }
opcode(ED, 48)    { OP_INC(cpu->bc.b.l); }
opcode(ED, 49)    { OP_OUTC(cpu->bc.b.l); }
opcode(ED, 4A)    { OP_ADC16(cpu->hl.w, cpu->bc.w); }
opcode(ED, 4B)    { OP_LD16FPCA(cpu->bc); }
opcode(ED, 4C)    { OP_NEG(); }
opcode(ED, 4D)    { OP_RETN(); }
opcode(ED, 4E)    { cpu->im = 0; }
opcode(ED, 4F)    { cpu->ir.b.l = cpu->af.b.h; cpu->r_top = cpu->af.b.h & 0x80; }
opcode(ED, 50)    { OP_INC(cpu->de.b.h); }
opcode(ED, 51)    { OP_OUTC(cpu->de.b.h); }
opcode(ED, 52)    { OP_SBC16(cpu->hl.w, cpu->de.w); }
opcode(ED, 53)    { OP_LD16TPCA(cpu->de); }
opcode(ED, 54)    { OP_NEG(); }
opcode(ED, 55)    { OP_RETN(); }
opcode(ED, 56)    { cpu->im = 1; }
opcode(ED, 57)    { OP_LDAI(); }
opcode(ED, 58)    { OP_INC(cpu->de.b.l); }
opcode(ED, 59)    { OP_OUTC(cpu->de.b.l); }
opcode(ED, 5A)    { OP_ADC16(cpu->hl.w, cpu->de.w); }
opcode(ED, 5B)    { OP_LD16FPCA(cpu->de); }
opcode(ED, 5C)    { OP_NEG(); }
opcode(ED, 5D)    { OP_RETN(); }
opcode(ED, 5E)    { cpu->im = 2; }
opcode(ED, 5F)    { OP_LDAR(); }
opcode(ED, 60)    { OP_INC(cpu->hl.b.h); }
opcode(ED, 61)    { OP_OUTC(cpu->hl.b.h); }
opcode(ED, 62)    { OP_SBC16(cpu->hl.w, cpu->hl.w); }
opcode(ED, 63)    { OP_LD16TPCA(cpu->hl); }
opcode(ED, 64)    { OP_NEG(); }
opcode(ED, 65)    { OP_RETN(); }
opcode(ED, 66)    { cpu->im = 0; }
opcode(ED, 67)    { OP_RRD(); }
opcode(ED, 68)    { OP_INC(cpu->hl.b.l); }
opcode(ED, 69)    { OP_OUTC(cpu->hl.b.l); }
opcode(ED, 6A)    { OP_ADC16(cpu->hl.w, cpu->hl.w); }
opcode(ED, 6B)    { OP_LD16FPCA(cpu->hl); }
opcode(ED, 6C)    { OP_NEG(); }
opcode(ED, 6D)    { OP_RETN(); }
opcode(ED, 6E)    { cpu->im = 0; }
opcode(ED, 6F)    { OP_RLD(); }
opcode(ED, 70)    { uint8 tmp; OP_INC(tmp); }
opcode(ED, 71)    { OP_OUTC(0); }
opcode(ED, 72)    { OP_SBC16(cpu->hl.w, cpu->sp.w); }
opcode(ED, 73)    { OP_LD16TPCA(cpu->sp); }
opcode(ED, 74)    { OP_NEG(); }
opcode(ED, 75)    { OP_RETN(); }
opcode(ED, 76)    { cpu->im = 1; }
opcode(ED, 77)    { }
opcode(ED, 78)    { OP_INC(cpu->af.b.h); }
opcode(ED, 79)    { OP_OUTC(cpu->af.b.h); }
opcode(ED, 7A)    { OP_ADC16(cpu->hl.w, cpu->sp.w); }
opcode(ED, 7B)    { OP_LD16FPCA(cpu->sp); }
opcode(ED, 7C)    { OP_NEG(); }
opcode(ED, 7D)    { OP_RETN(); }
opcode(ED, 7E)    { cpu->im = 1; }
opcode(ED, 7F)    { }
opcode(ED, 80)    { }
opcode(ED, 81)    { }
opcode(ED, 82)    { }
opcode(ED, 83)    { }
opcode(ED, 84)    { }
opcode(ED, 85)    { }
opcode(ED, 86)    { }
opcode(ED, 87)    { }
opcode(ED, 88)    { }
opcode(ED, 89)    { }
opcode(ED, 8A)    { }
opcode(ED, 8B)    { }
opcode(ED, 8C)    { }
opcode(ED, 8D)    { }
opcode(ED, 8E)    { }
opcode(ED, 8F)    { }
opcode(ED, 90)    { }
opcode(ED, 91)    { }
opcode(ED, 92)    { }
opcode(ED, 93)    { }
opcode(ED, 94)    { }
opcode(ED, 95)    { }
opcode(ED, 96)    { }
opcode(ED, 97)    { }
opcode(ED, 98)    { }
opcode(ED, 99)    { }
opcode(ED, 9A)    { }
opcode(ED, 9B)    { }
opcode(ED, 9C)    { }
opcode(ED, 9D)    { }
opcode(ED, 9E)    { }
opcode(ED, 9F)    { }
opcode(ED, A0)    { OP_LDI(); }
opcode(ED, A1)    { OP_CPI(); }
opcode(ED, A2)    { OP_INI(); }
opcode(ED, A3)    { OP_OUTI(); }
opcode(ED, A4)    { }
opcode(ED, A5)    { }
opcode(ED, A6)    { }
opcode(ED, A7)    { }
opcode(ED, A8)    { OP_LDD(); }
opcode(ED, A9)    { OP_CPD(); }
opcode(ED, AA)    { OP_IND(); }
opcode(ED, AB)    { OP_OUTD(); }
opcode(ED, AC)    { }
opcode(ED, AD)    { }
opcode(ED, AE)    { }
opcode(ED, AF)    { }
opcode(ED, B0)    { OP_LDIR(); }
opcode(ED, B1)    { OP_CPIR(); }
opcode(ED, B2)    { OP_INIR(); }
opcode(ED, B3)    { OP_OTIR(); }
opcode(ED, B4)    { }
opcode(ED, B5)    { }
opcode(ED, B6)    { }
opcode(ED, B7)    { }
opcode(ED, B8)    { OP_LDDR(); }
opcode(ED, B9)    { OP_CPDR(); }
opcode(ED, BA)    { OP_INDR(); }
opcode(ED, BB)    { OP_OTDR(); }
opcode(ED, BC)    { }
opcode(ED, BD)    { }
opcode(ED, BE)    { }
opcode(ED, BF)    { }
opcode(ED, C0)    { }
opcode(ED, C1)    { }
opcode(ED, C2)    { }
opcode(ED, C3)    { }
opcode(ED, C4)    { }
opcode(ED, C5)    { }
opcode(ED, C6)    { }
opcode(ED, C7)    { }
opcode(ED, C8)    { }
opcode(ED, C9)    { }
opcode(ED, CA)    { }
opcode(ED, CB)    { }
opcode(ED, CC)    { }
opcode(ED, CD)    { }
opcode(ED, CE)    { }
opcode(ED, CF)    { }
opcode(ED, D0)    { }
opcode(ED, D1)    { }
opcode(ED, D2)    { }
opcode(ED, D3)    { }
opcode(ED, D4)    { }
opcode(ED, D5)    { }
opcode(ED, D6)    { }
opcode(ED, D7)    { }
opcode(ED, D8)    { }
opcode(ED, D9)    { }
opcode(ED, DA)    { }
opcode(ED, DB)    { }
opcode(ED, DC)    { }
opcode(ED, DD)    { }
opcode(ED, DE)    { }
opcode(ED, DF)    { }
opcode(ED, E0)    { }
opcode(ED, E1)    { }
opcode(ED, E2)    { }
opcode(ED, E3)    { }
opcode(ED, E4)    { }
opcode(ED, E5)    { }
opcode(ED, E6)    { }
opcode(ED, E7)    { }
opcode(ED, E8)    { }
opcode(ED, E9)    { }
opcode(ED, EA)    { }
opcode(ED, EB)    { }
opcode(ED, EC)    { }
opcode(ED, ED)    { }
opcode(ED, EE)    { }
opcode(ED, EF)    { }
opcode(ED, F0)    { }
opcode(ED, F1)    { }
opcode(ED, F2)    { }
opcode(ED, F3)    { }
opcode(ED, F4)    { }
opcode(ED, F5)    { }
opcode(ED, F6)    { }
opcode(ED, F7)    { }
opcode(ED, F8)    { }
opcode(ED, F9)    { }
opcode(ED, FA)    { }
opcode(ED, FB)    { }
opcode(ED, FC)    { }
opcode(ED, FD)    { }
opcode(ED, FE)    { }
opcode(ED, FF)    { }

/* DDCB/FDCB prefixed opcodes */
opcode(DFDCB, 00)    { uint8 tmp = rbase(); OP_RLC(tmp); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 01)    { uint8 tmp = rbase(); OP_RLC(tmp); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 02)    { uint8 tmp = rbase(); OP_RLC(tmp); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 03)    { uint8 tmp = rbase(); OP_RLC(tmp); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 04)    { uint8 tmp = rbase(); OP_RLC(tmp); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 05)    { uint8 tmp = rbase(); OP_RLC(tmp); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 06)    { uint8 tmp = rbase(); OP_RLC(tmp); wbase(tmp); }
opcode(DFDCB, 07)    { uint8 tmp = rbase(); OP_RLC(tmp); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 08)    { uint8 tmp = rbase(); OP_RRC(tmp); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 09)    { uint8 tmp = rbase(); OP_RRC(tmp); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 0A)    { uint8 tmp = rbase(); OP_RRC(tmp); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 0B)    { uint8 tmp = rbase(); OP_RRC(tmp); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 0C)    { uint8 tmp = rbase(); OP_RRC(tmp); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 0D)    { uint8 tmp = rbase(); OP_RRC(tmp); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 0E)    { uint8 tmp = rbase(); OP_RRC(tmp); wbase(tmp); }
opcode(DFDCB, 0F)    { uint8 tmp = rbase(); OP_RRC(tmp); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 10)    { uint8 tmp = rbase(); OP_RL(tmp); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 11)    { uint8 tmp = rbase(); OP_RL(tmp); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 12)    { uint8 tmp = rbase(); OP_RL(tmp); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 13)    { uint8 tmp = rbase(); OP_RL(tmp); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 14)    { uint8 tmp = rbase(); OP_RL(tmp); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 15)    { uint8 tmp = rbase(); OP_RL(tmp); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 16)    { uint8 tmp = rbase(); OP_RL(tmp); wbase(tmp); }
opcode(DFDCB, 17)    { uint8 tmp = rbase(); OP_RL(tmp); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 18)    { uint8 tmp = rbase(); OP_RR(tmp); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 19)    { uint8 tmp = rbase(); OP_RR(tmp); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 1A)    { uint8 tmp = rbase(); OP_RR(tmp); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 1B)    { uint8 tmp = rbase(); OP_RR(tmp); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 1C)    { uint8 tmp = rbase(); OP_RR(tmp); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 1D)    { uint8 tmp = rbase(); OP_RR(tmp); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 1E)    { uint8 tmp = rbase(); OP_RR(tmp); wbase(tmp); }
opcode(DFDCB, 1F)    { uint8 tmp = rbase(); OP_RR(tmp); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 20)    { uint8 tmp = rbase(); OP_SLA(tmp); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 21)    { uint8 tmp = rbase(); OP_SLA(tmp); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 22)    { uint8 tmp = rbase(); OP_SLA(tmp); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 23)    { uint8 tmp = rbase(); OP_SLA(tmp); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 24)    { uint8 tmp = rbase(); OP_SLA(tmp); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 25)    { uint8 tmp = rbase(); OP_SLA(tmp); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 26)    { uint8 tmp = rbase(); OP_SLA(tmp); wbase(tmp); }
opcode(DFDCB, 27)    { uint8 tmp = rbase(); OP_SLA(tmp); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 28)    { uint8 tmp = rbase(); OP_SRA(tmp); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 29)    { uint8 tmp = rbase(); OP_SRA(tmp); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 2A)    { uint8 tmp = rbase(); OP_SRA(tmp); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 2B)    { uint8 tmp = rbase(); OP_SRA(tmp); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 2C)    { uint8 tmp = rbase(); OP_SRA(tmp); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 2D)    { uint8 tmp = rbase(); OP_SRA(tmp); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 2E)    { uint8 tmp = rbase(); OP_SRA(tmp); wbase(tmp); }
opcode(DFDCB, 2F)    { uint8 tmp = rbase(); OP_SRA(tmp); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 30)    { uint8 tmp = rbase(); OP_SLL(tmp); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 31)    { uint8 tmp = rbase(); OP_SLL(tmp); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 32)    { uint8 tmp = rbase(); OP_SLL(tmp); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 33)    { uint8 tmp = rbase(); OP_SLL(tmp); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 34)    { uint8 tmp = rbase(); OP_SLL(tmp); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 35)    { uint8 tmp = rbase(); OP_SLL(tmp); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 36)    { uint8 tmp = rbase(); OP_SLL(tmp); wbase(tmp); }
opcode(DFDCB, 37)    { uint8 tmp = rbase(); OP_SLL(tmp); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 38)    { uint8 tmp = rbase(); OP_SRL(tmp); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 39)    { uint8 tmp = rbase(); OP_SRL(tmp); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 3A)    { uint8 tmp = rbase(); OP_SRL(tmp); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 3B)    { uint8 tmp = rbase(); OP_SRL(tmp); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 3C)    { uint8 tmp = rbase(); OP_SRL(tmp); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 3D)    { uint8 tmp = rbase(); OP_SRL(tmp); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 3E)    { uint8 tmp = rbase(); OP_SRL(tmp); wbase(tmp); }
opcode(DFDCB, 3F)    { uint8 tmp = rbase(); OP_SRL(tmp); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 40)    { uint8 tmp = rbasei(); OP_BIT(tmp, 0); }
opcode(DFDCB, 41)    { uint8 tmp = rbasei(); OP_BIT(tmp, 0); }
opcode(DFDCB, 42)    { uint8 tmp = rbasei(); OP_BIT(tmp, 0); }
opcode(DFDCB, 43)    { uint8 tmp = rbasei(); OP_BIT(tmp, 0); }
opcode(DFDCB, 44)    { uint8 tmp = rbasei(); OP_BIT(tmp, 0); }
opcode(DFDCB, 45)    { uint8 tmp = rbasei(); OP_BIT(tmp, 0); }
opcode(DFDCB, 46)    { uint8 tmp = rbasei(); OP_BIT(tmp, 0); }
opcode(DFDCB, 47)    { uint8 tmp = rbasei(); OP_BIT(tmp, 0); }
opcode(DFDCB, 48)    { uint8 tmp = rbasei(); OP_BIT(tmp, 1); }
opcode(DFDCB, 49)    { uint8 tmp = rbasei(); OP_BIT(tmp, 1); }
opcode(DFDCB, 4A)    { uint8 tmp = rbasei(); OP_BIT(tmp, 1); }
opcode(DFDCB, 4B)    { uint8 tmp = rbasei(); OP_BIT(tmp, 1); }
opcode(DFDCB, 4C)    { uint8 tmp = rbasei(); OP_BIT(tmp, 1); }
opcode(DFDCB, 4D)    { uint8 tmp = rbasei(); OP_BIT(tmp, 1); }
opcode(DFDCB, 4E)    { uint8 tmp = rbasei(); OP_BIT(tmp, 1); }
opcode(DFDCB, 4F)    { uint8 tmp = rbasei(); OP_BIT(tmp, 1); }
opcode(DFDCB, 50)    { uint8 tmp = rbasei(); OP_BIT(tmp, 2); }
opcode(DFDCB, 51)    { uint8 tmp = rbasei(); OP_BIT(tmp, 2); }
opcode(DFDCB, 52)    { uint8 tmp = rbasei(); OP_BIT(tmp, 2); }
opcode(DFDCB, 53)    { uint8 tmp = rbasei(); OP_BIT(tmp, 2); }
opcode(DFDCB, 54)    { uint8 tmp = rbasei(); OP_BIT(tmp, 2); }
opcode(DFDCB, 55)    { uint8 tmp = rbasei(); OP_BIT(tmp, 2); }
opcode(DFDCB, 56)    { uint8 tmp = rbasei(); OP_BIT(tmp, 2); }
opcode(DFDCB, 57)    { uint8 tmp = rbasei(); OP_BIT(tmp, 2); }
opcode(DFDCB, 58)    { uint8 tmp = rbasei(); OP_BIT(tmp, 3); }
opcode(DFDCB, 59)    { uint8 tmp = rbasei(); OP_BIT(tmp, 3); }
opcode(DFDCB, 5A)    { uint8 tmp = rbasei(); OP_BIT(tmp, 3); }
opcode(DFDCB, 5B)    { uint8 tmp = rbasei(); OP_BIT(tmp, 3); }
opcode(DFDCB, 5C)    { uint8 tmp = rbasei(); OP_BIT(tmp, 3); }
opcode(DFDCB, 5D)    { uint8 tmp = rbasei(); OP_BIT(tmp, 3); }
opcode(DFDCB, 5E)    { uint8 tmp = rbasei(); OP_BIT(tmp, 3); }
opcode(DFDCB, 5F)    { uint8 tmp = rbasei(); OP_BIT(tmp, 3); }
opcode(DFDCB, 60)    { uint8 tmp = rbasei(); OP_BIT(tmp, 4); }
opcode(DFDCB, 61)    { uint8 tmp = rbasei(); OP_BIT(tmp, 4); }
opcode(DFDCB, 62)    { uint8 tmp = rbasei(); OP_BIT(tmp, 4); }
opcode(DFDCB, 63)    { uint8 tmp = rbasei(); OP_BIT(tmp, 4); }
opcode(DFDCB, 64)    { uint8 tmp = rbasei(); OP_BIT(tmp, 4); }
opcode(DFDCB, 65)    { uint8 tmp = rbasei(); OP_BIT(tmp, 4); }
opcode(DFDCB, 66)    { uint8 tmp = rbasei(); OP_BIT(tmp, 4); }
opcode(DFDCB, 67)    { uint8 tmp = rbasei(); OP_BIT(tmp, 4); }
opcode(DFDCB, 68)    { uint8 tmp = rbasei(); OP_BIT(tmp, 5); }
opcode(DFDCB, 69)    { uint8 tmp = rbasei(); OP_BIT(tmp, 5); }
opcode(DFDCB, 6A)    { uint8 tmp = rbasei(); OP_BIT(tmp, 5); }
opcode(DFDCB, 6B)    { uint8 tmp = rbasei(); OP_BIT(tmp, 5); }
opcode(DFDCB, 6C)    { uint8 tmp = rbasei(); OP_BIT(tmp, 5); }
opcode(DFDCB, 6D)    { uint8 tmp = rbasei(); OP_BIT(tmp, 5); }
opcode(DFDCB, 6E)    { uint8 tmp = rbasei(); OP_BIT(tmp, 5); }
opcode(DFDCB, 6F)    { uint8 tmp = rbasei(); OP_BIT(tmp, 5); }
opcode(DFDCB, 70)    { uint8 tmp = rbasei(); OP_BIT(tmp, 6); }
opcode(DFDCB, 71)    { uint8 tmp = rbasei(); OP_BIT(tmp, 6); }
opcode(DFDCB, 72)    { uint8 tmp = rbasei(); OP_BIT(tmp, 6); }
opcode(DFDCB, 73)    { uint8 tmp = rbasei(); OP_BIT(tmp, 6); }
opcode(DFDCB, 74)    { uint8 tmp = rbasei(); OP_BIT(tmp, 6); }
opcode(DFDCB, 75)    { uint8 tmp = rbasei(); OP_BIT(tmp, 6); }
opcode(DFDCB, 76)    { uint8 tmp = rbasei(); OP_BIT(tmp, 6); }
opcode(DFDCB, 77)    { uint8 tmp = rbasei(); OP_BIT(tmp, 6); }
opcode(DFDCB, 78)    { uint8 tmp = rbasei(); OP_BIT(tmp, 7); }
opcode(DFDCB, 79)    { uint8 tmp = rbasei(); OP_BIT(tmp, 7); }
opcode(DFDCB, 7A)    { uint8 tmp = rbasei(); OP_BIT(tmp, 7); }
opcode(DFDCB, 7B)    { uint8 tmp = rbasei(); OP_BIT(tmp, 7); }
opcode(DFDCB, 7C)    { uint8 tmp = rbasei(); OP_BIT(tmp, 7); }
opcode(DFDCB, 7D)    { uint8 tmp = rbasei(); OP_BIT(tmp, 7); }
opcode(DFDCB, 7E)    { uint8 tmp = rbasei(); OP_BIT(tmp, 7); }
opcode(DFDCB, 7F)    { uint8 tmp = rbasei(); OP_BIT(tmp, 7); }
opcode(DFDCB, 80)    { uint8 tmp = rbase(); OP_RES(tmp, 0); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 81)    { uint8 tmp = rbase(); OP_RES(tmp, 0); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 82)    { uint8 tmp = rbase(); OP_RES(tmp, 0); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 83)    { uint8 tmp = rbase(); OP_RES(tmp, 0); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 84)    { uint8 tmp = rbase(); OP_RES(tmp, 0); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 85)    { uint8 tmp = rbase(); OP_RES(tmp, 0); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 86)    { uint8 tmp = rbase(); OP_RES(tmp, 0); wbase(tmp); }
opcode(DFDCB, 87)    { uint8 tmp = rbase(); OP_RES(tmp, 0); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 88)    { uint8 tmp = rbase(); OP_RES(tmp, 1); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 89)    { uint8 tmp = rbase(); OP_RES(tmp, 1); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 8A)    { uint8 tmp = rbase(); OP_RES(tmp, 1); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 8B)    { uint8 tmp = rbase(); OP_RES(tmp, 1); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 8C)    { uint8 tmp = rbase(); OP_RES(tmp, 1); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 8D)    { uint8 tmp = rbase(); OP_RES(tmp, 1); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 8E)    { uint8 tmp = rbase(); OP_RES(tmp, 1); wbase(tmp); }
opcode(DFDCB, 8F)    { uint8 tmp = rbase(); OP_RES(tmp, 1); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 90)    { uint8 tmp = rbase(); OP_RES(tmp, 2); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 91)    { uint8 tmp = rbase(); OP_RES(tmp, 2); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 92)    { uint8 tmp = rbase(); OP_RES(tmp, 2); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 93)    { uint8 tmp = rbase(); OP_RES(tmp, 2); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 94)    { uint8 tmp = rbase(); OP_RES(tmp, 2); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 95)    { uint8 tmp = rbase(); OP_RES(tmp, 2); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 96)    { uint8 tmp = rbase(); OP_RES(tmp, 2); wbase(tmp); }
opcode(DFDCB, 97)    { uint8 tmp = rbase(); OP_RES(tmp, 2); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, 98)    { uint8 tmp = rbase(); OP_RES(tmp, 3); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, 99)    { uint8 tmp = rbase(); OP_RES(tmp, 3); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, 9A)    { uint8 tmp = rbase(); OP_RES(tmp, 3); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, 9B)    { uint8 tmp = rbase(); OP_RES(tmp, 3); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, 9C)    { uint8 tmp = rbase(); OP_RES(tmp, 3); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, 9D)    { uint8 tmp = rbase(); OP_RES(tmp, 3); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, 9E)    { uint8 tmp = rbase(); OP_RES(tmp, 3); wbase(tmp); }
opcode(DFDCB, 9F)    { uint8 tmp = rbase(); OP_RES(tmp, 3); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, A0)    { uint8 tmp = rbase(); OP_RES(tmp, 4); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, A1)    { uint8 tmp = rbase(); OP_RES(tmp, 4); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, A2)    { uint8 tmp = rbase(); OP_RES(tmp, 4); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, A3)    { uint8 tmp = rbase(); OP_RES(tmp, 4); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, A4)    { uint8 tmp = rbase(); OP_RES(tmp, 4); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, A5)    { uint8 tmp = rbase(); OP_RES(tmp, 4); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, A6)    { uint8 tmp = rbase(); OP_RES(tmp, 4); wbase(tmp); }
opcode(DFDCB, A7)    { uint8 tmp = rbase(); OP_RES(tmp, 4); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, A8)    { uint8 tmp = rbase(); OP_RES(tmp, 5); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, A9)    { uint8 tmp = rbase(); OP_RES(tmp, 5); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, AA)    { uint8 tmp = rbase(); OP_RES(tmp, 5); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, AB)    { uint8 tmp = rbase(); OP_RES(tmp, 5); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, AC)    { uint8 tmp = rbase(); OP_RES(tmp, 5); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, AD)    { uint8 tmp = rbase(); OP_RES(tmp, 5); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, AE)    { uint8 tmp = rbase(); OP_RES(tmp, 5); wbase(tmp); }
opcode(DFDCB, AF)    { uint8 tmp = rbase(); OP_RES(tmp, 5); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, B0)    { uint8 tmp = rbase(); OP_RES(tmp, 6); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, B1)    { uint8 tmp = rbase(); OP_RES(tmp, 6); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, B2)    { uint8 tmp = rbase(); OP_RES(tmp, 6); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, B3)    { uint8 tmp = rbase(); OP_RES(tmp, 6); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, B4)    { uint8 tmp = rbase(); OP_RES(tmp, 6); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, B5)    { uint8 tmp = rbase(); OP_RES(tmp, 6); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, B6)    { uint8 tmp = rbase(); OP_RES(tmp, 6); wbase(tmp); }
opcode(DFDCB, B7)    { uint8 tmp = rbase(); OP_RES(tmp, 6); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, B8)    { uint8 tmp = rbase(); OP_RES(tmp, 7); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, B9)    { uint8 tmp = rbase(); OP_RES(tmp, 7); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, BA)    { uint8 tmp = rbase(); OP_RES(tmp, 7); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, BB)    { uint8 tmp = rbase(); OP_RES(tmp, 7); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, BC)    { uint8 tmp = rbase(); OP_RES(tmp, 7); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, BD)    { uint8 tmp = rbase(); OP_RES(tmp, 7); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, BE)    { uint8 tmp = rbase(); OP_RES(tmp, 7); wbase(tmp); }
opcode(DFDCB, BF)    { uint8 tmp = rbase(); OP_RES(tmp, 7); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, C0)    { uint8 tmp = rbase(); OP_SET(tmp, 0); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, C1)    { uint8 tmp = rbase(); OP_SET(tmp, 0); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, C2)    { uint8 tmp = rbase(); OP_SET(tmp, 0); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, C3)    { uint8 tmp = rbase(); OP_SET(tmp, 0); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, C4)    { uint8 tmp = rbase(); OP_SET(tmp, 0); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, C5)    { uint8 tmp = rbase(); OP_SET(tmp, 0); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, C6)    { uint8 tmp = rbase(); OP_SET(tmp, 0); wbase(tmp); }
opcode(DFDCB, C7)    { uint8 tmp = rbase(); OP_SET(tmp, 0); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, C8)    { uint8 tmp = rbase(); OP_SET(tmp, 1); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, C9)    { uint8 tmp = rbase(); OP_SET(tmp, 1); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, CA)    { uint8 tmp = rbase(); OP_SET(tmp, 1); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, CB)    { uint8 tmp = rbase(); OP_SET(tmp, 1); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, CC)    { uint8 tmp = rbase(); OP_SET(tmp, 1); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, CD)    { uint8 tmp = rbase(); OP_SET(tmp, 1); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, CE)    { uint8 tmp = rbase(); OP_SET(tmp, 1); wbase(tmp); }
opcode(DFDCB, CF)    { uint8 tmp = rbase(); OP_SET(tmp, 1); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, D0)    { uint8 tmp = rbase(); OP_SET(tmp, 2); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, D1)    { uint8 tmp = rbase(); OP_SET(tmp, 2); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, D2)    { uint8 tmp = rbase(); OP_SET(tmp, 2); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, D3)    { uint8 tmp = rbase(); OP_SET(tmp, 2); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, D4)    { uint8 tmp = rbase(); OP_SET(tmp, 2); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, D5)    { uint8 tmp = rbase(); OP_SET(tmp, 2); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, D6)    { uint8 tmp = rbase(); OP_SET(tmp, 2); wbase(tmp); }
opcode(DFDCB, D7)    { uint8 tmp = rbase(); OP_SET(tmp, 2); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, D8)    { uint8 tmp = rbase(); OP_SET(tmp, 3); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, D9)    { uint8 tmp = rbase(); OP_SET(tmp, 3); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, DA)    { uint8 tmp = rbase(); OP_SET(tmp, 3); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, DB)    { uint8 tmp = rbase(); OP_SET(tmp, 3); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, DC)    { uint8 tmp = rbase(); OP_SET(tmp, 3); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, DD)    { uint8 tmp = rbase(); OP_SET(tmp, 3); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, DE)    { uint8 tmp = rbase(); OP_SET(tmp, 3); wbase(tmp); }
opcode(DFDCB, DF)    { uint8 tmp = rbase(); OP_SET(tmp, 3); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, E0)    { uint8 tmp = rbase(); OP_SET(tmp, 4); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, E1)    { uint8 tmp = rbase(); OP_SET(tmp, 4); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, E2)    { uint8 tmp = rbase(); OP_SET(tmp, 4); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, E3)    { uint8 tmp = rbase(); OP_SET(tmp, 4); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, E4)    { uint8 tmp = rbase(); OP_SET(tmp, 4); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, E5)    { uint8 tmp = rbase(); OP_SET(tmp, 4); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, E6)    { uint8 tmp = rbase(); OP_SET(tmp, 4); wbase(tmp); }
opcode(DFDCB, E7)    { uint8 tmp = rbase(); OP_SET(tmp, 4); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, E8)    { uint8 tmp = rbase(); OP_SET(tmp, 5); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, E9)    { uint8 tmp = rbase(); OP_SET(tmp, 5); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, EA)    { uint8 tmp = rbase(); OP_SET(tmp, 5); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, EB)    { uint8 tmp = rbase(); OP_SET(tmp, 5); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, EC)    { uint8 tmp = rbase(); OP_SET(tmp, 5); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, ED)    { uint8 tmp = rbase(); OP_SET(tmp, 5); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, EE)    { uint8 tmp = rbase(); OP_SET(tmp, 5); wbase(tmp); }
opcode(DFDCB, EF)    { uint8 tmp = rbase(); OP_SET(tmp, 5); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, F0)    { uint8 tmp = rbase(); OP_SET(tmp, 6); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, F1)    { uint8 tmp = rbase(); OP_SET(tmp, 6); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, F2)    { uint8 tmp = rbase(); OP_SET(tmp, 6); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, F3)    { uint8 tmp = rbase(); OP_SET(tmp, 6); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, F4)    { uint8 tmp = rbase(); OP_SET(tmp, 6); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, F5)    { uint8 tmp = rbase(); OP_SET(tmp, 6); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, F6)    { uint8 tmp = rbase(); OP_SET(tmp, 6); wbase(tmp); }
opcode(DFDCB, F7)    { uint8 tmp = rbase(); OP_SET(tmp, 6); wbase(tmp); cpu->af.b.h = tmp; }
opcode(DFDCB, F8)    { uint8 tmp = rbase(); OP_SET(tmp, 7); wbase(tmp); cpu->bc.b.h = tmp; }
opcode(DFDCB, F9)    { uint8 tmp = rbase(); OP_SET(tmp, 7); wbase(tmp); cpu->bc.b.l = tmp; }
opcode(DFDCB, FA)    { uint8 tmp = rbase(); OP_SET(tmp, 7); wbase(tmp); cpu->de.b.h = tmp; }
opcode(DFDCB, FB)    { uint8 tmp = rbase(); OP_SET(tmp, 7); wbase(tmp); cpu->de.b.l = tmp; }
opcode(DFDCB, FC)    { uint8 tmp = rbase(); OP_SET(tmp, 7); wbase(tmp); cpu->hl.b.h = tmp; }
opcode(DFDCB, FD)    { uint8 tmp = rbase(); OP_SET(tmp, 7); wbase(tmp); cpu->hl.b.l = tmp; }
opcode(DFDCB, FE)    { uint8 tmp = rbase(); OP_SET(tmp, 7); wbase(tmp); }
opcode(DFDCB, FF)    { uint8 tmp = rbase(); OP_SET(tmp, 7); wbase(tmp); cpu->af.b.h = tmp; }
