/*
	This file is part of CrabEmu.

	Copyright (C) 2005, 2006 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#define OP_LD16FPC(val)   {   \
    (val).b.l = cpu->mread(cpu->pc.w++);  \
    (val).b.h = cpu->mread(cpu->pc.w++);  \
}

#define OP_LD16TPCA(val)    {   \
    uint16 _tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8); \
    cpu->mwrite(_tmp + 1, (val).b.h); \
    cpu->mwrite(_tmp, (val).b.l); \
}

#define OP_LD16FPCA(val)    {   \
    uint16 _tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8); \
    (val).b.l = cpu->mread(_tmp); \
    (val).b.h = cpu->mread(_tmp + 1); \
}

#define OP_LD8TPCA(val) {   \
    uint16 _tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8); \
    cpu->mwrite(_tmp, (val)); \
}

#define OP_LD8FPCA(val) {   \
    uint16 _tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8); \
    (val) = cpu->mread(_tmp); \
}

#define OP_LD8TMEM(reg, addr)   {   \
    cpu->mwrite((addr), (reg)); \
}

#define OP_LD8FMEM(reg, addr)  {   \
    (reg) = cpu->mread((addr)); \
}

#define OP_LD8MTM() {   \
    int8 tmp = (int8) cpu->mread(cpu->pc.w++); \
    cpu->mwrite(tmp + cpu->offset->w, cpu->mread(cpu->pc.w++)); \
}

#define OP_LDAI()   {   \
    cpu->af.b.h = cpu->ir.b.h; \
    cpu->af.b.l = (cpu->ir.b.h & 0x80) | (cpu->ir.b.h ? 0x00 : 0x40) | \
        (cpu->iff2 << 2) | (cpu->af.b.l & 0x01); \
}

#define OP_LDAR()   {   \
    cpu->af.b.h = (cpu->ir.b.l & 0x7F) | cpu->r_top; \
    cpu->af.b.l = ZStable[cpu->af.b.h] | (cpu->iff2 << 2) | \
        (cpu->af.b.l & 0x01); \
}

#define OP_INC16(val)   {   \
    ++val;  \
}

#define OP_INC8(val)    {   \
    uint32 _tmp = (val) + 1; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | \
        ((_tmp & 0x0F) ? 0x00 : 0x10) | \
        (((val) == 0x7F) ? 0x04: 0x00) | \
        (cpu->af.b.l & 0x01); \
    (val) = _tmp; \
}

#define OP_DEC16(val)   {   \
    --val;  \
}

#define OP_DEC8(val)    {   \
    uint32 _tmp = (val) - 1; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | \
        ((val) == 0x80 ? 0x04 : 0x00) | \
        (cpu->af.b.l & 0x01) | \
        (((val) ^ _tmp) & 0x10) | \
        0x02; \
    (val) = _tmp; \
}

#define OP_ADD8(val)    {   \
    uint8 _value = (val); \
    uint32 _tmp = cpu->af.b.h + _value; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | \
        ((cpu->af.b.h ^ _tmp ^ _value) & 0x10) | \
        (((_value ^ cpu->af.b.h ^ 0x80) & (_value ^ _tmp) & 0x80) >> 5) | \
        ((_tmp & 0x100) >> 8); \
    cpu->af.b.h = _tmp;  \
}

#define OP_ADD16(reg1, reg2)    {   \
    uint32 _tmp = (reg1).w + (reg2).w; \
    cpu->af.b.l = (cpu->af.b.l & 0xC4) | \
        ((((reg1).w ^ _tmp ^ (reg2).w) & 0x1000) >> 8) | \
        ((_tmp >> 16) & 0x01); \
    (reg1).w = _tmp; \
}

#define OP_ADC8(val)    {   \
    uint8 _value = (val); \
    uint32 _tmp = cpu->af.b.h + _value + (cpu->af.b.l & 0x01); \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | \
        ((cpu->af.b.h ^ _tmp ^ _value) & 0x10) | \
        (((_value ^ cpu->af.b.h ^ 0x80) & (_value ^ _tmp) & 0x80) >> 5) | \
        ((_tmp & 0x100) >> 8); \
    cpu->af.b.h = _tmp; \
}

#define OP_ADC16(reg1, reg2)    {   \
    uint32 _tmp = (reg1) + (reg2) + (cpu->af.b.l & 0x01); \
    cpu->af.b.l = ((_tmp & 0x8000) >> 8) | ((_tmp & 0xFFFF) ? 0 : 0x40) | \
        ((((reg1) ^ _tmp ^ (reg2)) & 0x1000) >> 8) | \
        ((((reg2) ^ (reg1) ^ 0x8000) & ((reg2) ^ _tmp) & 0x8000) >> 13) | \
        ((_tmp >> 16) & 0x01); \
    (reg1) = _tmp; \
}

#define OP_SUB8(val)    {   \
    uint8 _value = (val); \
    uint32 _tmp = cpu->af.b.h - _value; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | \
        ((cpu->af.b.h ^ _tmp ^ _value) & 0x10) | \
        (((_value ^ cpu->af.b.h) & (cpu->af.b.h ^ _tmp) & 0x80) >> 5) | \
        0x02 | \
        ((_tmp & 0x100) >> 8); \
    cpu->af.b.h = _tmp; \
}

#define OP_SBC8(val)    {   \
    uint8 _value = (val); \
    uint32 _tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - _value; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | \
        ((cpu->af.b.h ^ _tmp ^ _value) & 0x10) | \
        (((_value ^ cpu->af.b.h) & (cpu->af.b.h ^ _tmp) & 0x80) >> 5) | \
        0x02 | \
        ((_tmp >> 8) & 0x01); \
    cpu->af.b.h = _tmp; \
}

#define OP_SBC16(reg1, reg2)    { \
    uint32 _tmp = (reg1) - (cpu->af.b.l & 0x01) - (reg2); \
    cpu->af.b.l = ((_tmp >> 8) & 0x80) | ((_tmp & 0xFFFF) ? 0x00 : 0x40) | \
        ((((reg1) ^ _tmp ^ (reg2)) >> 8) & 0x10) | \
        ((((reg2) ^ (reg1)) & ((reg1) ^ _tmp) & 0x8000) >> 13) | 0x02 | \
        ((_tmp >> 16) & 0x01); \
    (reg1) = _tmp; \
}

#define OP_AND8(val)    {   \
    uint32 _tmp = cpu->af.b.h & (val); \
    cpu->af.b.l = ZSPtable[_tmp & 0xFF] | \
        0x10; \
    cpu->af.b.h = _tmp; \
}

#define OP_XOR8(val)    {   \
    uint32 _tmp = cpu->af.b.h ^ (val); \
    cpu->af.b.l = ZSPtable[_tmp & 0xFF]; \
    cpu->af.b.h = _tmp; \
}

#define OP_OR8(val) {   \
    uint32 _tmp = cpu->af.b.h | (val); \
    cpu->af.b.l = ZSPtable[_tmp & 0xFF]; \
    cpu->af.b.h = _tmp; \
}

#define OP_CP8(val) {   \
    uint8 _value = (val); \
    uint32 _tmp = cpu->af.b.h - _value; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | \
        ((cpu->af.b.h ^ _tmp ^ _value) & 0x10) | \
        (((_value ^ cpu->af.b.h) & (cpu->af.b.h ^ _tmp) & 0x80) >> 5) | \
        0x02 | \
        ((_tmp & 0x100) >> 8); \
}

#define OP_PUSH16(val)  {   \
    cpu->mwrite(--cpu->sp.w, (val).b.h); \
    cpu->mwrite(--cpu->sp.w, (val).b.l); \
}

#define OP_POP16(reg)   {   \
    (reg).b.l = cpu->mread(cpu->sp.w++); \
    (reg).b.h = cpu->mread(cpu->sp.w++); \
}

#define OP_RST(val) {   \
    OP_PUSH16(cpu->pc); \
    cpu->pc.w = val; \
}

#define OP_RLCA()   {   \
    cpu->af.b.h = (cpu->af.b.h << 1) | (cpu->af.b.h >> 7); \
    cpu->af.b.l = (cpu->af.b.l & 0xC4) | (cpu->af.b.h & 0x01); \
}

#define OP_RRCA()   {   \
    cpu->af.b.l = (cpu->af.b.l & 0xC4) | (cpu->af.b.h & 0x01); \
    cpu->af.b.h = (cpu->af.b.h >> 1) | (cpu->af.b.h << 7); \
}

#define OP_RLA()    {   \
    uint8 _tmp = (cpu->af.b.h << 1) | (cpu->af.b.l & 0x01); \
    cpu->af.b.l = (cpu->af.b.l & 0xC4) | (cpu->af.b.h >> 7); \
    cpu->af.b.h = _tmp; \
}

#define OP_RRA()    {   \
    uint8 _tmp = (cpu->af.b.h >> 1) | (cpu->af.b.l << 7); \
    cpu->af.b.l = (cpu->af.b.l & 0xC4) | (cpu->af.b.h & 0x01); \
    cpu->af.b.h = _tmp; \
}

#define OP_DAA()    {   \
    int _index = cpu->af.b.h; \
    _index |= ((cpu->af.b.l & 0x01) ? 0x100 : 0x000) | \
        ((cpu->af.b.l & 0x10) ? 0x200 : 0x000) | \
        ((cpu->af.b.l & 0x02) ? 0x400 : 0x000); \
    cpu->af.w = DAAtable[_index]; \
}

#define OP_CPL()    {   \
    cpu->af.b.h = ~cpu->af.b.h; \
    cpu->af.b.l |= 0x12; \
}

#define OP_SCF()    {   \
    cpu->af.b.l = (cpu->af.b.l & 0xC4) | 0x01; \
}

#define OP_CCF()    {   \
    cpu->af.b.l = (cpu->af.b.l & 0xC4) | ((cpu->af.b.l & 0x01) ? 0x10 : 0x01); \
}

#define OP_EX(reg1, reg2)   {   \
    uint16 _tmp = (reg1).w; \
    (reg1).w = (reg2).w; \
    (reg2).w = _tmp;  \
}

#define OP_EXX()    {   \
    OP_EX(cpu->bc, cpu->bcp); \
    OP_EX(cpu->de, cpu->dep); \
    OP_EX(cpu->hl, cpu->hlp); \
}

#define OP_EXSP(reg) {   \
    uint16 _tmp = (reg).w; \
    (reg).b.l = cpu->mread(cpu->sp.w++); \
    (reg).b.h = cpu->mread(cpu->sp.w); \
    cpu->mwrite(cpu->sp.w, (_tmp >> 8)); \
    cpu->mwrite(--cpu->sp.w, _tmp); \
}

#define OP_JR() {   \
    cpu->pc.w += (int8) cpu->mread(cpu->pc.w) + 1; \
}

#define OP_CONDJR(cond) {   \
    if((cond))  { \
        cpu->pc.w += (int8) cpu->mread(cpu->pc.w) + 1; \
        cpu->cycles += 5; \
    } \
    else    { \
        ++cpu->pc.w; \
    } \
}

#define OP_RET()    {   \
    cpu->pc.b.l = cpu->mread(cpu->sp.w++); \
    cpu->pc.b.h = cpu->mread(cpu->sp.w++); \
}

#define OP_CONDRET(cond)    {   \
    if((cond))  { \
        OP_RET(); \
        cpu->cycles += 6; \
    } \
}

#define OP_CALL()   {   \
    uint16 _tmp = (cpu->mread(cpu->pc.w++)) | (cpu->mread(cpu->pc.w++) << 8); \
    cpu->mwrite(--cpu->sp.w, cpu->pc.b.h); \
    cpu->mwrite(--cpu->sp.w, cpu->pc.b.l); \
    cpu->pc.w = _tmp; \
}

#define OP_CONDCALL(cond)   {   \
    if((cond))  {   \
        OP_CALL(); \
        cpu->cycles += 7; \
    } \
    else    { \
        cpu->pc.w += 2; \
    } \
}

#define OP_JP() {   \
    cpu->pc.w = (cpu->mread(cpu->pc.w++)) | (cpu->mread(cpu->pc.w) << 8); \
}

#define OP_CONDJP(cond) {   \
    if((cond))  { \
        OP_JP(); \
    } \
    else    { \
        cpu->pc.w += 2; \
    } \
}

#define OP_INPC()   {   \
    cpu->af.b.h = cpu->pread(cpu->mread(cpu->pc.w++)); \
}

#define OP_INC(reg)    {   \
    (reg) = cpu->pread(cpu->bc.b.l); \
    cpu->af.b.l = ZSPtable[(reg)] | (cpu->af.b.l & 0x01); \
}
    

#define OP_OUTPC()    {   \
    cpu->pwrite(cpu->mread(cpu->pc.w++), cpu->af.b.h); \
}

#define OP_OUTC(reg)    { \
    cpu->pwrite(cpu->bc.b.l, (reg)); \
}

#define OP_EI() {   \
    if(cpu->iff1)   { \
        cpu->iff2 = 1; \
    } \
    else    { \
        cpu->iff1 = cpu->iff2 = 1; \
        cpu->ei = 1; \
        ++cpu->ir.b.l; \
    } \
}

#define OP_HALT()   {   \
    --cpu->pc.w; \
    cpu->halt = 1; \
}

#define OP_RLC(reg) {   \
    uint8 _tmp = ((reg) << 1) | (((reg) & 0x80) >> 7); \
    cpu->af.b.l = ZSPtable[_tmp] | (((reg) & 0x80) >> 7); \
    (reg) = _tmp; \
}

#define OP_RRC(reg) {   \
    uint8 _tmp = ((reg) >> 1) | (((reg) & 0x01) << 7); \
    cpu->af.b.l = ZSPtable[_tmp] | ((reg) & 0x01); \
    (reg) = _tmp; \
}

#define OP_RL(reg)  {   \
    uint8 _tmp = ((reg) << 1) | (cpu->af.b.l & 0x01); \
    cpu->af.b.l = ZSPtable[_tmp] | (((reg) & 0x80) >> 7); \
    (reg) = _tmp; \
}

#define OP_RR(reg)  {   \
    uint8 _tmp = ((reg) >> 1) | ((cpu->af.b.l & 0x01) << 7); \
    cpu->af.b.l = ZSPtable[_tmp] | ((reg) & 0x01); \
    (reg) = _tmp; \
}

#define OP_SLA(reg) {   \
    uint8 _tmp = ((reg) << 1); \
    cpu->af.b.l = ZSPtable[_tmp] | (((reg) & 0x80) >> 7); \
    (reg) = _tmp; \
}

#define OP_SRA(reg) {   \
    uint8 _tmp = ((reg) >> 1) | ((reg) & 0x80); \
    cpu->af.b.l = ZSPtable[_tmp] | ((reg) & 0x01); \
    (reg) = _tmp; \
}

#define OP_SLL(reg) {   \
    uint8 _tmp = ((reg) << 1) | 0x01; \
    cpu->af.b.l = ZSPtable[_tmp] | (((reg) & 0x80) >> 7); \
    (reg) = _tmp; \
}

#define OP_SRL(reg) {   \
    uint8 _tmp = (reg) >> 1; \
    cpu->af.b.l = ZSPtable[_tmp] | ((reg) & 0x01); \
    (reg) = _tmp; \
}

#define OP_RRD()    {   \
    uint8 _byte = cpu->mread(cpu->hl.w); \
    uint8 _tmp = ((cpu->af.b.h & 0x0F) << 4) | ((_byte & 0xF0) >> 4); \
    cpu->af.b.h = (cpu->af.b.h & 0xF0) | (_byte & 0x0F); \
    cpu->af.b.l = ZSPtable[cpu->af.b.h] | (cpu->af.b.l & 0x01); \
    cpu->mwrite(cpu->hl.w, _tmp); \
}

#define OP_RLD()    {   \
    uint8 _byte = cpu->mread(cpu->hl.w); \
    uint8 _tmp = ((_byte & 0x0F) << 4) | (cpu->af.b.h & 0x0F); \
    cpu->af.b.h = (cpu->af.b.h & 0xF0) | ((_byte & 0xF0) >> 4); \
    cpu->af.b.l = ZSPtable[cpu->af.b.h] | (cpu->af.b.l & 0x01); \
    cpu->mwrite(cpu->hl.w, _tmp); \
}

#define OP_BIT(reg, bit)    {   \
    cpu->af.b.l = ZSPtable[(reg) & (1 << (bit))] | 0x10 | (cpu->af.b.l & 0x01); \
}

#define OP_RES(reg, bit)    {   \
    (reg) &= ~(1 << (bit)); \
}

#define OP_SET(reg, bit)    {   \
    (reg) |= (1 << (bit)); \
}

#define OP_LDI()    {   \
    cpu->mwrite(cpu->de.w++, cpu->mread(cpu->hl.w++)); \
    --cpu->bc.w; \
    cpu->af.b.l = (cpu->af.b.l & 0xC1) | (cpu->bc.w ? 0x04 : 0x00); \
}

#define OP_LDIR()   {   \
    uint8 _byte = cpu->mread(cpu->hl.w++); \
    cpu->mwrite(cpu->de.w++, _byte); \
    --cpu->bc.w; \
    cpu->af.b.l = (cpu->af.b.l & 0xC1) | (cpu->bc.w ? 0x04 : 0x00); \
    if(cpu->bc.w)   { \
        cpu->pc.w -= 2; \
        cpu->cycles += 5; \
    } \
}

#define OP_LDD()    {   \
    cpu->mwrite(cpu->de.w--, cpu->mread(cpu->hl.w--)); \
    --cpu->bc.w; \
    cpu->af.b.l = (cpu->af.b.l & 0xC1) | (cpu->bc.w ? 0x04 : 0x00); \
}

#define OP_LDDR()   {   \
    cpu->mwrite(cpu->de.w--, cpu->mread(cpu->hl.w--)); \
    --cpu->bc.w; \
    cpu->af.b.l = (cpu->af.b.l & 0xC1) | (cpu->bc.w ? 0x04 : 0x00); \
    if(cpu->bc.w)	{ \
        cpu->pc.w -= 2; \
        cpu->cycles += 5; \
    } \
}

#define OP_CPI()    {   \
    uint8 _byte = cpu->mread(cpu->hl.w++); \
    uint32 _tmp = cpu->af.b.h - _byte; \
    --cpu->bc.w; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | ((cpu->af.b.h ^ _tmp ^ _byte) & 0x10) | \
        (cpu->bc.w ? 0x04 : 0x00) | 0x02 | (cpu->af.b.l & 0x01); \
}

#define OP_CPIR()   {   \
    uint8 _byte = cpu->mread(cpu->hl.w++); \
    uint32 _tmp = cpu->af.b.h - _byte; \
    --cpu->bc.w; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | ((_byte ^ cpu->af.b.h ^ _tmp) & 0x10) | \
        (cpu->bc.w ? 0x04 : 0x00) | 0x02 | (cpu->af.b.l & 0x01); \
    if(cpu->bc.w && _tmp)	{ \
        cpu->pc.w -= 2; \
        cpu->cycles += 5; \
    } \
}

#define OP_CPD()    {   \
    uint8 _byte = cpu->mread(cpu->hl.w--); \
    uint32 _tmp = cpu->af.b.h - _byte; \
    --cpu->bc.w; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | ((_byte ^ cpu->af.b.h ^ _tmp) & 0x10) | \
        (cpu->bc.w ? 0x04 : 0x00) | 0x02 | (cpu->af.b.l & 0x01); \
}

#define OP_CPDR()   {   \
    uint8 _byte = cpu->mread(cpu->hl.w--); \
    uint32 _tmp = cpu->af.b.h - _byte; \
    --cpu->bc.w; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | ((_byte ^ cpu->af.b.h ^ _tmp) & 0x10) | \
        (cpu->bc.w ? 0x04 : 0x00) | 0x02 | (cpu->af.b.l & 0x01); \
    if(cpu->bc.w && _tmp)	{ \
        cpu->pc.w -= 2; \
        cpu->cycles += 5; \
    } \
}

#define OP_INI()    {   \
    uint8 _byte = cpu->pread(cpu->bc.b.l); \
    cpu->mwrite(cpu->hl.w++, _byte); \
    --cpu->bc.b.h; \
    cpu->af.b.l = ZStable[cpu->bc.b.h] | \
        ((_byte + ((cpu->bc.b.l + 1) & 0xFF)) > 0xFF ? 0x11 : 0x00) | \
        (ZSPtable[((_byte + ((cpu->bc.b.l + 1) & 0xFF)) & 0x07) ^ cpu->bc.b.h] & 0x04) | \
        ((_byte >> 6) & 0x02); \
}

#define OP_INIR()   {   \
    uint8 _byte = cpu->pread(cpu->bc.b.l); \
    cpu->mwrite(cpu->hl.w++, _byte); \
    --cpu->bc.b.h; \
    cpu->af.b.l = ZStable[cpu->bc.b.h] | \
        ((_byte + ((cpu->bc.b.l + 1) & 0xFF)) > 0xFF ? 0x11 : 0x00) | \
        (ZSPtable[((_byte + ((cpu->bc.b.l + 1) & 0xFF)) & 0x07) ^ cpu->bc.b.h] & 0x04) | \
        ((_byte >> 6) & 0x02); \
    if(cpu->bc.b.h)	{ \
        cpu->pc.w -= 2; \
        cpu->cycles += 5; \
    } \
}

#define OP_IND()    {   \
    uint8 _byte = cpu->pread(cpu->bc.b.l); \
    cpu->mwrite(cpu->hl.w--, _byte); \
    --cpu->bc.b.h; \
    cpu->af.b.l = ZStable[cpu->bc.b.h] | \
        ((_byte + ((cpu->bc.b.l - 1) & 0xFF)) > 0xFF ? 0x11 : 0x00) | \
        (ZSPtable[((_byte + ((cpu->bc.b.l - 1) & 0xFF)) & 0x07) ^ cpu->bc.b.h] & 0x04) | \
        ((_byte >> 6) & 0x02); \
}

#define OP_INDR()   {   \
    uint8 _byte = cpu->pread(cpu->bc.b.l); \
    cpu->mwrite(cpu->hl.w--, _byte); \
    --cpu->bc.b.h; \
    cpu->af.b.l = ZStable[cpu->bc.b.h] | \
        ((_byte + ((cpu->bc.b.l - 1) & 0xFF)) > 0xFF ? 0x11 : 0x00) | \
        (ZSPtable[((_byte + ((cpu->bc.b.l - 1) & 0xFF)) & 0x07) ^ cpu->bc.b.h] & 0x04) | \
        ((_byte >> 6) & 0x02); \
    if(cpu->bc.b.h) { \
        cpu->pc.w -= 2; \
        cpu->cycles += 5; \
    } \
}

#define OP_OUTI()   {   \
    uint8 _byte = cpu->mread(cpu->hl.w++); \
    --cpu->bc.b.h; \
    cpu->pwrite(cpu->bc.b.l, _byte); \
    cpu->af.b.l = ZStable[cpu->bc.b.h] | ((cpu->hl.b.l + _byte) > 0xFF ? 0x11 : 0x00) | \
        (ZSPtable[((cpu->hl.b.l + _byte) & 0x07) ^ cpu->bc.b.h] & 0x04) | \
        ((_byte >> 6) & 0x02); \
}

#define OP_OTIR()   {   \
    uint8 _byte = cpu->mread(cpu->hl.w++); \
    --cpu->bc.b.h; \
    cpu->pwrite(cpu->bc.b.l, _byte); \
    cpu->af.b.l = ZStable[cpu->bc.b.h] | \
        ((cpu->hl.b.l + _byte) > 0xFF ? 0x11 : 0x00) | \
        (ZSPtable[((cpu->hl.b.l + _byte) & 0x07) ^ cpu->bc.b.h] & 0x04) | \
        ((_byte >> 6) & 0x02); \
    if(cpu->bc.b.h)	{ \
        cpu->pc.w -= 2; \
        cpu->cycles += 5; \
    } \
}

#define OP_OUTD()   {   \
    uint8 _byte = cpu->mread(cpu->hl.w--); \
    --cpu->bc.b.h; \
    cpu->pwrite(cpu->bc.b.l, _byte); \
    cpu->af.b.l = ZStable[cpu->bc.b.h] | \
        ((cpu->hl.b.l + _byte) > 0xFF ? 0x11 : 0x00) | \
        (ZSPtable[((cpu->hl.b.l + _byte) & 0x07) ^ cpu->bc.b.h] & 0x04) | \
        ((_byte >> 6) & 0x02); \
}

#define OP_OTDR()   {   \
    uint8 _byte = cpu->mread(cpu->hl.w--); \
    --cpu->bc.b.h; \
    cpu->pwrite(cpu->bc.b.l, _byte); \
    cpu->af.b.l = ZStable[cpu->bc.b.h] | \
        ((cpu->hl.b.l + _byte) > 0xFF ? 0x11 : 0x00) | \
        (ZSPtable[((cpu->hl.b.l + _byte) & 0x07) ^ cpu->bc.b.h] & 0x04) | \
        ((_byte >> 6) & 0x02); \
    if(cpu->bc.b.h) { \
        cpu->pc.w -= 2; \
        cpu->cycles += 5; \
    } \
}

#define OP_NEG()    {   \
    uint32 _tmp = 0 - cpu->af.b.h; \
    cpu->af.b.l = ZStable[_tmp & 0xFF] | ((0 ^ _tmp ^ cpu->af.b.h) & 0x10) | \
        (((cpu->af.b.h ^ 0) & (0 ^ _tmp) & 0x80) >> 5) | 0x02 | \
        ((_tmp >> 8) & 0x01); \
    cpu->af.b.h = _tmp; \
}

#define OP_RETN()   {   \
    cpu->pc.b.l = cpu->mread(cpu->sp.w++); \
    cpu->pc.b.h = cpu->mread(cpu->sp.w++); \
    cpu->iff1 = cpu->iff2; \
}
