/*
	This file is part of CrabEmu.

	Copyright (C) 2005, 2006 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "EmuController.h"
#include "EmuOGLView.h"
#include "sms.h"
#include "smsmem.h"
#include "sn76489.h"

/* From main.m */
extern NSLock *memoryLock;
extern EmuOGLView *theview;
extern void emu_loop();

/* from sms.c */
extern int sms_psg_enabled;
extern sn76489_t psg;
extern int sms_initialized;

int started = 0;

@implementation EmuController

- (void)awakeFromNib
{
    prefs = [[NSUserDefaults standardUserDefaults] retain];

    if([prefs objectForKey: @"SMS Buttons"] != nil) {
        buttonmap = [NSString stringWithString: [prefs objectForKey: @"SMS Buttons"]];
        [buttonmap retain];
    }
    else    {
        unichar string[14];

        string[0] = 0x7E;
        string[1] = 0x7D;
        string[2] = 0x7B;
        string[3] = 0x7C;
        string[4] = 0x29;
        string[5] = 0x27;
        string[6] = 0x23;
        string[7] = 0x0D;
        string[8] = 0x01;
        string[9] = 0x00;
        string[10] = 0x02;
        string[11] = 0x09;
        string[12] = 0x0B;
        string[13] = 0x69;

        buttonmap = [NSString stringWithCharacters: string length: 14];
        [buttonmap retain];
        
        [prefs setObject: buttonmap forKey: @"SMS Buttons"];
        [prefs synchronize];
    }
}

- (void)dealloc
{
    [prefs setObject: buttonmap forKey: @"SMS Buttons"];
    [prefs synchronize];
    [prefs release];
    [buttonmap release];

    [super dealloc];
}

- (IBAction)enableNoise:(id)sender
{
	if([noise state] == NSOnState)	{
		[noise setState: NSOffState];
		psg.enabled_channels &= ~ENABLE_NOISE;
	}
	else	{
		[noise setState: NSOnState];
		psg.enabled_channels |= ENABLE_NOISE;
	}
}

- (IBAction)enablePSG:(id)sender
{
	if([PSG state] == NSOnState)	{
		[PSG setState: NSOffState];
		sms_psg_enabled = 0;
	}
	else	{
		[PSG setState: NSOnState];
		sms_psg_enabled = 1;
	}
}

- (IBAction)enableTone0:(id)sender
{
	if([tone0 state] == NSOnState)	{
		[tone0 setState: NSOffState];
		psg.enabled_channels &= ~ENABLE_TONE0;
	}
	else	{
		[tone0 setState: NSOnState];
		psg.enabled_channels |= ENABLE_TONE0;
	}
}

- (IBAction)enableTone1:(id)sender
{
	if([tone1 state] == NSOnState)	{
		[tone1 setState: NSOffState];
		psg.enabled_channels &= ~ENABLE_TONE1;
	}
	else	{
		[tone1 setState: NSOnState];
		psg.enabled_channels |= ENABLE_TONE1;
	}
}

- (IBAction)enableTone2:(id)sender
{
	if([tone2 state] == NSOnState)	{
		[tone2 setState: NSOffState];
		psg.enabled_channels &= ~ENABLE_TONE2;
	}
	else	{
		[tone2 setState: NSOnState];
		psg.enabled_channels |= ENABLE_TONE2;
	}
}

- (IBAction)loadMasterRom:(id)sender
{
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	[openPanel setTitle: @"Choose a Master System/Game Gear Rom"];
	[openPanel setCanChooseDirectories: NO];
	
	if([openPanel runModal] == NSOKButton)	{
		NSString *filePath = [openPanel filename];

        [memoryLock lock];
        if(sms_initialized == 1)    {
            sms_reset();
        }
        else    {
            sms_init();
        }

		sms_mem_load_rom([filePath UTF8String]);
        [memoryLock unlock];
        [theview showWindow];

        if(started == 0)    {
            [NSThread detachNewThreadSelector:@selector(masterEmuThread:) toTarget:self withObject:nil];
            started = 1;
        }
	}
}

- (IBAction)prefGGStart:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[6] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms1B1:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];

    buttons[4] = [self runVirtKeyboard];

    [buttonmap release];

    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms1B2:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[5] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms1Down:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[1] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms1Left:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[2] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms1Reset:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[0] = 0x7E;
    buttons[1] = 0x7D;
    buttons[2] = 0x7B;
    buttons[3] = 0x7C;
    buttons[4] = 0x29;
    buttons[5] = 0x27;
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}    

- (IBAction)prefSms1Right:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[3] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms1Up:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[0] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms2B1:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[11] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms2B2:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[12] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms2Down:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[8] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms2Left:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[9] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms2Reset:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[7] = 0x0D;
    buttons[8] = 0x01;
    buttons[9] = 0x00;
    buttons[10] = 0x02;
    buttons[11] = 0x09;
    buttons[12] = 0x0B;
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms2Right:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[10] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (IBAction)prefSms2Up:(id)sender
{
    unichar buttons[14];
    [buttonmap getCharacters: buttons];
    
    buttons[7] = [self runVirtKeyboard];
    
    [buttonmap release];
    
    buttonmap = [NSString stringWithCharacters: buttons length: 14];
    [buttonmap retain];
}

- (unichar)runVirtKeyboard
{
    unichar retval;

    [NSApp beginSheet: virtKeyboard modalForWindow: prefWindow modalDelegate: self didEndSelector: NULL contextInfo: NULL];

    retval = (unichar) [NSApp runModalForWindow: virtKeyboard];

    [NSApp endSheet: virtKeyboard];
    [virtKeyboard orderOut: nil];

    return retval;
}

- (void)endVirtKeyboard:(unichar)character
{
    [NSApp stopModalWithCode: (int)character];
}

- (void)masterEmuThread:(id)junk
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	emu_loop();

	sms_shutdown();
	[pool release];
}

- (void)pauseEmulation:(id)sender
{
    if(started == 0)
        return;

    if([pause state] == NSOffState) {
        [pause setState: NSOnState];
        [memoryLock lock];
    }
    else    {
        [pause setState: NSOffState];
        [memoryLock unlock];
    }
}

- (void)updateButtonMapFromString: (NSString *)str
{
    [str retain];
    [buttonmap release];
    buttonmap = str;
}

- (NSString *)buttonMap
{
    return buttonmap;
}

@end
