/*
	This file is part of CrabEmu.

	Copyright (C) 2005, 2006 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "EmuView.h"
#include "EmuController.h"

#include "sms.h"
#include "smsvdp.h"

extern EmuView *theview;
extern sms_vdp_t smsvdp;

@implementation EmuView

- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil) {
		theview = self;
	}

	return self;
}

- (void)drawRect:(NSRect)rect
{
	CGContextRef cgContext;
	CGImageRef cgImage;
	CGColorSpaceRef cgSpace;
	CGDataProviderRef cgData;
	CGRect cgRect;
	unsigned char *bitmap;

	cgContext = [[NSGraphicsContext currentContext] graphicsPort];
	bitmap = smsvdp.framebuffer;
	cgData = CGDataProviderCreateWithData(NULL, bitmap, 256 * 192 * 4, NULL);
	cgSpace = CGColorSpaceCreateDeviceRGB();
	cgRect = CGRectMake(0, 0, 256, 192);
	cgImage = CGImageCreate(256, 192, 8, 32, 256 * 4, cgSpace, kCGImageAlphaNoneSkipLast, cgData, NULL, 0, kCGRenderingIntentDefault);
	CGContextDrawImage(cgContext, cgRect, cgImage);
	CGDataProviderRelease(cgData);
	CGColorSpaceRelease(cgSpace);
	CGImageRelease(cgImage);
}

- (void)resizeWindowWithViewWidth: (float)width height: (float)height
{
    NSRect frame = [self frame];
    float deltax, deltay;

    deltax = width - frame.size.width;
    deltay = height - frame.size.height;

    frame.size.width = width;
    frame.size.height = height;

    [self setFrame: frame];

    frame = [window frame];

    frame.size.width += deltax;
    frame.size.height += deltay;
    frame.origin.y -= deltay;

    [window setFrame: frame display: NO];
}

- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (void)showWindow
{
    [window makeKeyAndOrderFront:self];
}

- (void)keyDown:(NSEvent *)event
{
    unichar buttonmap[14];
	unichar character = (unichar)[event keyCode];

    [[controller buttonMap] getCharacters: buttonmap];

	if(character == buttonmap[0])
        sms_button_pressed(SMS_PAD1_UP);
    else if(character == buttonmap[1])
        sms_button_pressed(SMS_PAD1_DOWN);
    else if(character == buttonmap[2])
        sms_button_pressed(SMS_PAD1_LEFT);
    else if(character == buttonmap[3])
        sms_button_pressed(SMS_PAD1_RIGHT);
    else if(character == buttonmap[4])
        sms_button_pressed(SMS_PAD1_A);
    else if(character == buttonmap[5])
        sms_button_pressed(SMS_PAD1_B);
    else if(character == buttonmap[6])
        sms_button_pressed(GG_START);
    else if(character == buttonmap[7])
        sms_button_pressed(SMS_PAD2_UP);
    else if(character == buttonmap[8])
        sms_button_pressed(SMS_PAD2_DOWN);
    else if(character == buttonmap[9])
        sms_button_pressed(SMS_PAD2_LEFT);
    else if(character == buttonmap[10])
        sms_button_pressed(SMS_PAD2_RIGHT);
    else if(character == buttonmap[11])
        sms_button_pressed(SMS_PAD2_A);
    else if(character == buttonmap[12])
        sms_button_pressed(SMS_PAD2_B);
    else
        [super keyDown: event];
}

- (void)keyUp:(NSEvent *)event
{
    unichar buttonmap[14];
	unichar character = (unichar)[event keyCode];
    
    [[controller buttonMap] getCharacters: buttonmap];
    
	if(character == buttonmap[0])
        sms_button_released(SMS_PAD1_UP);
    else if(character == buttonmap[1])
        sms_button_released(SMS_PAD1_DOWN);
    else if(character == buttonmap[2])
        sms_button_released(SMS_PAD1_LEFT);
    else if(character == buttonmap[3])
        sms_button_released(SMS_PAD1_RIGHT);
    else if(character == buttonmap[4])
        sms_button_released(SMS_PAD1_A);
    else if(character == buttonmap[5])
        sms_button_released(SMS_PAD1_B);
    else if(character == buttonmap[6])
        sms_button_released(GG_START);
    else if(character == buttonmap[7])
        sms_button_released(SMS_PAD2_UP);
    else if(character == buttonmap[8])
        sms_button_released(SMS_PAD2_DOWN);
    else if(character == buttonmap[9])
        sms_button_released(SMS_PAD2_LEFT);
    else if(character == buttonmap[10])
        sms_button_released(SMS_PAD2_RIGHT);
    else if(character == buttonmap[11])
        sms_button_released(SMS_PAD2_A);
    else if(character == buttonmap[12])
        sms_button_released(SMS_PAD2_B);
    else
        [super keyUp: event];
}

@end
