/*
    This file is part of CrabEmu.

    Copyright (C) 2005, 2006 Lawrence Sebald

    CrabEmu is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 
    as published by the Free Software Foundation.

    CrabEmu is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CrabEmu; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef SN76489_H
#define SN76489_H

typedef struct sn76489_struct   {
    unsigned char volume[4];
    unsigned short tone[3];
    unsigned char noise;

    unsigned short noise_shift;
    unsigned short noise_initial_shift;
    unsigned short noise_tapped;

    signed char tone_state[4];

    unsigned char latched_reg;

    float counter[4];

    unsigned char enabled_channels;

    float clocks_per_sample;
} sn76489_t;

#define LATCH_TONE0 0x00
#define LATCH_TONE1 0x20
#define LATCH_TONE2 0x40
#define LATCH_NOISE 0x60
#define LATCH_VOL0 0x10
#define LATCH_VOL1 0x30
#define LATCH_VOL2 0x50
#define LATCH_VOL3 0x70

#define ENABLE_TONE0 0x01
#define ENABLE_TONE1 0x02
#define ENABLE_TONE2 0x04
#define ENABLE_NOISE 0x08

int sn76489_init(sn76489_t *psg, float clock, float sample_rate,
                 unsigned short noise_shift, unsigned short tapped);
void sn76489_write(sn76489_t *psg, unsigned char byte);
unsigned long sn76489_execute(sn76489_t *psg, signed short *buf,
                              unsigned long cpu_cycles);
void sn76489_execute_samples(sn76489_t *psg, signed short *buf,
                             unsigned long samples);

#endif
