/*
	This file is part of CrabEmu.

	Copyright (C) 2005, 2006, 2007 Lawrence Sebald

    CrabEmu is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 
    as published by the Free Software Foundation.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "EmuOGLView.h"
#include "EmuController.h"

#include <OpenGL/GL.h>

#include "sms.h"
#include "smsvdp.h"

extern EmuOGLView *theview;
extern void render_scene();

@implementation EmuOGLView

- (id)initWithFrame:(NSRect)frameRect
{
    NSOpenGLPixelFormatAttribute attrs[] = {
        NSOpenGLPFAWindow,
        NSOpenGLPFASingleRenderer,
        NSOpenGLPFANoRecovery,
        NSOpenGLPFAColorSize, 24,
        NSOpenGLPFADepthSize, 16,
        NSOpenGLPFADoubleBuffer,
        NSOpenGLPFAAccelerated,
        NSOpenGLPFAScreenMask, CGDisplayIDToOpenGLDisplayMask(kCGDirectMainDisplay),
        0
    };

    NSOpenGLPixelFormat *fmt = [[NSOpenGLPixelFormat alloc] initWithAttributes: attrs];
    if(fmt == nil)  {
        return nil;
    }
    [fmt autorelease];

    if ((self = [super initWithFrame: frameRect pixelFormat: fmt]) != nil) {
        theview = self;
    }
    else    {
        return nil;
    }

    doubleSize = NO;

    return self;
}

- (void)drawRect:(NSRect)rect
{
    render_scene();

    [[NSOpenGLContext currentContext] flushBuffer];
}

- (void)resizeWindowWithViewWidth:(float)w height:(float)h
{
    NSRect frame = [self frame];
    float deltax, deltay;

    width = w;
    height = h;

    if(doubleSize == YES)   {
        w *= 2;
        h *= 2;

        deltax = w - frame.size.width;
        deltay = h - frame.size.height;

        frame.size.width = w;
        frame.size.height = h;
    }
    else    {
        deltax = w - frame.size.width;
        deltay = h - frame.size.height;

        frame.size.width = w;
        frame.size.height = h;
    }

    [self setFrame: frame];

    frame = [window frame];

    frame.size.width += deltax;
    frame.size.height += deltay;
    frame.origin.y -= deltay;

    [window setFrame: frame display: NO];
}

- (void)setDoubleSize:(BOOL) ds
{
    if((doubleSize == NO && ds == YES) ||
       (doubleSize == YES && ds == NO)) {
        doubleSize = ds;

        [self resizeWindowWithViewWidth:width
                                 height:height];
    }
}

- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (void)showWindow
{
    [window makeKeyAndOrderFront:self];
}

- (void)keyDown:(NSEvent *)event
{
    unichar buttonmap[14];
    unichar character = (unichar)[event keyCode];

    [[controller buttonMap] getCharacters: buttonmap];

    if(character == buttonmap[0])
        sms_button_pressed(SMS_PAD1_UP);
    else if(character == buttonmap[1])
        sms_button_pressed(SMS_PAD1_DOWN);
    else if(character == buttonmap[2])
        sms_button_pressed(SMS_PAD1_LEFT);
    else if(character == buttonmap[3])
        sms_button_pressed(SMS_PAD1_RIGHT);
    else if(character == buttonmap[4])
        sms_button_pressed(SMS_PAD1_A);
    else if(character == buttonmap[5])
        sms_button_pressed(SMS_PAD1_B);
    else if(character == buttonmap[6])
        sms_button_pressed(GG_START);
    else if(character == buttonmap[7])
        sms_button_pressed(SMS_PAD2_UP);
    else if(character == buttonmap[8])
        sms_button_pressed(SMS_PAD2_DOWN);
    else if(character == buttonmap[9])
        sms_button_pressed(SMS_PAD2_LEFT);
    else if(character == buttonmap[10])
        sms_button_pressed(SMS_PAD2_RIGHT);
    else if(character == buttonmap[11])
        sms_button_pressed(SMS_PAD2_A);
    else if(character == buttonmap[12])
        sms_button_pressed(SMS_PAD2_B);
    else
        [super keyDown: event];
}

- (void)keyUp:(NSEvent *)event
{
    unichar buttonmap[14];
    unichar character = (unichar)[event keyCode];

    [[controller buttonMap] getCharacters: buttonmap];

    if(character == buttonmap[0])
        sms_button_released(SMS_PAD1_UP);
    else if(character == buttonmap[1])
        sms_button_released(SMS_PAD1_DOWN);
    else if(character == buttonmap[2])
        sms_button_released(SMS_PAD1_LEFT);
    else if(character == buttonmap[3])
        sms_button_released(SMS_PAD1_RIGHT);
    else if(character == buttonmap[4])
        sms_button_released(SMS_PAD1_A);
    else if(character == buttonmap[5])
        sms_button_released(SMS_PAD1_B);
    else if(character == buttonmap[6])
        sms_button_released(GG_START);
    else if(character == buttonmap[7])
        sms_button_released(SMS_PAD2_UP);
    else if(character == buttonmap[8])
        sms_button_released(SMS_PAD2_DOWN);
    else if(character == buttonmap[9])
        sms_button_released(SMS_PAD2_LEFT);
    else if(character == buttonmap[10])
        sms_button_released(SMS_PAD2_RIGHT);
    else if(character == buttonmap[11])
        sms_button_released(SMS_PAD2_A);
    else if(character == buttonmap[12])
        sms_button_released(SMS_PAD2_B);
    else
        [super keyUp: event];
}

- (float) width
{
    return width;
}

- (float) height
{
    return height;
}

- (BOOL)doubleSize
{
    return doubleSize;
}

@end
