/*
    This file is part of CrabEmu.

    Copyright (C) 2007 Lawrence Sebald

    CrabEmu is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 
    as published by the Free Software Foundation.

    CrabEmu is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CrabEmu; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <dc/sound/stream.h>
#include <arch/spinlock.h>
#include <string.h>
#include "sound.h"

#define UNUSED __attribute__((unused))

static snd_stream_hnd_t stream;
static unsigned char buffer[44100 * 20 * 2];
static int buffer_size = 16384;
static spinlock_t lock = SPINLOCK_INITIALIZER;
static unsigned char buf_out[8192];

void *sound_callback(snd_stream_hnd_t hnd UNUSED, int smp_req, int *smp_rcv)    {
    spinlock_lock(&lock);

    if(buffer_size < smp_req)   {
        *smp_rcv = buffer_size;
        memcpy(buf_out, buffer, buffer_size);
        buffer_size = 0;
    }
    else    {
        *smp_rcv = smp_req;
        memcpy(buf_out, buffer, smp_req);
        buffer_size -= smp_req;

        if(buffer_size)
            memcpy(buffer, buffer + smp_req, buffer_size);
    }

    spinlock_unlock(&lock);

    return buf_out;
}

void sound_update_buffer(signed short *buf, int length) {
#if 0
    spinlock_lock(&lock);

    memcpy(buffer + buffer_size, buf, length);
    buffer_size += length;

    spinlock_unlock(&lock);

    snd_stream_poll(stream);
#endif
}

int sound_init(void)    {
#if 0
    memset(buffer, 0, 44100 * 20 * 2);

    if(snd_stream_init())   {
        return -1;
    }

    stream = snd_stream_alloc(sound_callback, 8192);

    snd_stream_start(stream, 44100, 1);
#endif
    return 0;
}

void sound_shutdown(void)   {
#if 0
    snd_stream_stop(stream);
    snd_stream_shutdown();
#endif
}

void sound_reset_buffer(void)   {
    memset(buffer, 0, 16384);
    buffer_size = 16384;
}
